from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Awp0FullTextSavedSearch
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class FullTextSavedSearchInput(TcBaseObj):
    """
    Input structure of createFullTextSavedSearch SOA
    
    :var clientId: Client Id
    :var savedSearchName: Saved search name
    :var searchString: Search string
    :var searchFilterMap: Search filter map
    """
    clientId: str = ''
    savedSearchName: str = ''
    searchString: str = ''
    searchFilterMap: SearchFilterMap = None


@dataclass
class FullTextSearchInput(TcBaseObj):
    """
    A structure containing input search criteria.
    
    :var searchString: The string to search for.
    :var startIndex: The index from which to start the search.
    :var maxToReturn: The maximum number of objects to return.
    :var maxToLoad: The maximum number of objects to load.
    :var searchFilterMap: The map containing the list of search filters for each search filter field.
    :var searchSortCriteria: The criteria to use to sort the results.
    :var searchFilterFieldSortType: The sorting type to use to order the search filter categories in the response. The
    acceptable values are: "Alphabetical", "Priority".
    :var attributesToInflate: The list of attributes to inflate.
    """
    searchString: str = ''
    startIndex: int = 0
    maxToReturn: int = 0
    maxToLoad: int = 0
    searchFilterMap: SearchFilterMap = None
    searchSortCriteria: List[SearchSortCriteria] = ()
    searchFilterFieldSortType: SearchFilterFieldSortType = None
    attributesToInflate: List[str] = ()


@dataclass
class FullTextSearchResponse(TcBaseObj):
    """
    A service response structure containing search results.
    
    :var searchResults: The list of business objects obtained after performing a search.
    :var totalFound: The total number of business objects found.
    :var totalLoaded: The total number of business objects loaded.
    :var searchFilterMap: The map containing the list of search filters for each search filter field based on the
    search results.
    :var searchFilterCategories: A list of search filter categories ordered by filter priority.
    :var defaultFilterFieldDisplayCount: The default number of search filter categories to display.
    :var serviceData: The service data object.
    """
    searchResults: List[BusinessObject] = ()
    totalFound: int = 0
    totalLoaded: int = 0
    searchFilterMap: SearchFilterMap = None
    searchFilterCategories: List[SearchFilterField] = ()
    defaultFilterFieldDisplayCount: int = 0
    serviceData: ServiceData = None


@dataclass
class GetAddedObjectsToUpdateIndexResponse(TcBaseObj):
    """
    The service response structure containing UIDs of added objects to update the full text search index.
    
    :var uidsAddedObjects: The list containing UIDs of added objects to update the full text search index.
    :var serviceData: The service data object.
    """
    uidsAddedObjects: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class GetAddedObjectsToUpdateIndexResponse1(TcBaseObj):
    """
    The service response structure containing UIDs of added objects to update the full text search index. Also includes
    the date and time at which the query is executed to find the data.
    
    :var uidsAddedObjects: The list containing UIDs of added objects to update the full text search index.
    :var queriedDateTime: The data and time at which the added objects are queried for. Require this to keep track of
    sync case.
    :var serviceData: the service data object.
    """
    uidsAddedObjects: List[str] = ()
    queriedDateTime: datetime = None
    serviceData: ServiceData = None


@dataclass
class GetDatasetIndexableFilesInfoResponse(TcBaseObj):
    """
    The response structure with dataset(s) file information map.
    
    :var dsFileInfosMap: Map conatining datasets file information.
    :var serviceData: Service Data
    """
    dsFileInfosMap: DatasetFileTicketInfosMap = None
    serviceData: ServiceData = None


@dataclass
class GetDeletedObjectsToUpdateIndexResponse(TcBaseObj):
    """
    The service response structure containing UIDs of deleted objects to update the full text search index.
    
    :var uidsDeletedObjects: The list containing UIDs of deleted objects to update the full text search index.
    :var serviceData: The service data object.
    """
    uidsDeletedObjects: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class GetDeletedObjectsToUpdateIndexResponse1(TcBaseObj):
    """
    The service response structure containing UIDs of deleted objects to update the full text search index.  Also
    contains the date and time at which the deleted objects are queried for.
    
    :var uidsDeletedObjects: List of UIDs of deleted objects to update the full text search index.
    :var queriedDateTime: Date including time at which the deleted objects are queried for. This value will be used for
    sync use cases.
    :var serviceData: The service data object.
    """
    uidsDeletedObjects: List[str] = ()
    queriedDateTime: datetime = None
    serviceData: ServiceData = None


@dataclass
class GetImpactedItemRevsForReIndexResponse(TcBaseObj):
    """
    The service response structure holds the impacted ItemRevision UIDs for which the revision rule selectors
    information need to be updated in full text search index.
    
    :var impactedObjectsUIDs: List that contains the impacted ItemRevision UIDs due to adds/modify/deletes that
    happened after last time index.
    :var serviceData: The Service Data
    """
    impactedObjectsUIDs: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class GetObjectsToIndexResponse(TcBaseObj):
    """
    A response structure containing a list of objects to index.
    
    :var objectsToIndexResults: The list of objects to index.
    :var serviceData: The service data object.
    """
    objectsToIndexResults: List[ObjectsToIndexResult] = ()
    serviceData: ServiceData = None


@dataclass
class GetTcObjectsToIndexInput(TcBaseObj):
    """
    The input to get the objects to index.
    
    :var dateTimeRanges: The date/time ranges to query on.
    """
    dateTimeRanges: List[DateTimeRange] = ()


@dataclass
class ObjectsToIndexResult(TcBaseObj):
    """
    A structure containing the list of objects to index within a date/time range.
    
    :var dateTimeRange: The date/time range for the list of objects returned.
    :var objectUIDsForRange: The list of object UIDs within a specific date/time range.
    """
    dateTimeRange: DateTimeRange = None
    objectUIDsForRange: List[str] = ()


@dataclass
class PreFiltersProperty(TcBaseObj):
    """
    Pre filters property internal, display name and its values.
    
    :var internalName: PreFilter Property internal name.
    :var displayName: PreFilters property display name.
    :var values: Vector of PreFilter values
    """
    internalName: str = ''
    displayName: str = ''
    values: List[PreFiltersValue] = ()


@dataclass
class PreFiltersResponse(TcBaseObj):
    """
    List of PreFilters and their values.
    
    :var properties: Vector of PreFilter properties.
    :var serviceData: The service data object.
    """
    properties: List[PreFiltersProperty] = ()
    serviceData: ServiceData = None


@dataclass
class PreFiltersValue(TcBaseObj):
    """
    Internal and display value of the prefilter.
    
    :var internalName: Filter value internal name.
    :var displayName: Filter value display name.
    """
    internalName: str = ''
    displayName: str = ''


@dataclass
class SearchFilter(TcBaseObj):
    """
    A structure representing a string, date or numeric type of search filter.
    
    :var searchFilterType: The type of search filter. Valid values are "StringFilter", "DateFilter", "NumericFilter".
    :var stringValue: The value for a string filter. This field is applicable only if the "searchFilterType" field is
    set to "StringFilter".
    :var startDateValue: The starting value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter".
    :var endDateValue: The ending value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter".
    :var startNumericValue: The starting value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var endNumericValue: The ending value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var count: The number of values in the filter. This field is populated on the service response and is ignored on
    the service input.
    :var selected: A flag that indicates if the filter was previously selected and used to filter the search results.
    This field is populated on the service response and is ignored on the service input.
    :var startEndRange: The 'gap' used to generate the start and end values
    """
    searchFilterType: SearchFilterType = None
    stringValue: str = ''
    startDateValue: datetime = None
    endDateValue: datetime = None
    startNumericValue: float = 0.0
    endNumericValue: float = 0.0
    count: int = 0
    selected: bool = False
    startEndRange: str = ''


@dataclass
class SearchFilterField(TcBaseObj):
    """
    A structure containing details about a search filter field.
    
    :var internalName: The internal name for the search filter field.
    :var displayName: The display name for the search filter field.
    :var defaultFilterValueDisplayCount: The default number of search filter values to display within the search filter
    field.
    """
    internalName: str = ''
    displayName: str = ''
    defaultFilterValueDisplayCount: int = 0


@dataclass
class SearchSortCriteria(TcBaseObj):
    """
    The criteria to use to sort the results.
    
    :var fieldName: The name of the field on which to perform the sorting.
    :var sortDirection: The direction in which the sorting needs to be perfomed - ascending or descending.
    """
    fieldName: str = ''
    sortDirection: SortDirection = None


@dataclass
class SearchSuggestionsInput(TcBaseObj):
    """
    A structure containing input data to retrieve search suggestions.
    
    :var searchString: The string to search for.
    :var maxCount: The maximum number of suggestions to return.
    """
    searchString: str = ''
    maxCount: int = 0


@dataclass
class SearchSuggestionsResponse(TcBaseObj):
    """
    A response structure containing a list of suggestions.
    
    :var suggestions: The list of search suggestions.
    :var serviceData: The service data object.
    """
    suggestions: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class UpdateIndexerStatusInput(TcBaseObj):
    """
    Input required to keep track of the exported data for further sync cases.
    
    :var applicationId: Application ID used to perform the action during Export/Sync process.
    :var lastProcessedDate: Last proceesed date of the action.
    """
    applicationId: str = ''
    lastProcessedDate: datetime = None


@dataclass
class CreateFullTextSavedSearchResponse(TcBaseObj):
    """
    Response from createFullTextSavedSearch SOA
    
    :var output: Map from client Id to created FullTextSavedSearch object
    :var serviceData: Service data
    """
    output: CreateFullTextSavedSearchOutput = None
    serviceData: ServiceData = None


@dataclass
class DatasetFileInfo(TcBaseObj):
    """
    Structure containing the datasets file information.
    
    :var fileUID: Unique Identifier of the file.
    :var fmsFileTicket: FMS read file ticket for the file to be downloaded.
    :var fileName: Original File name
    :var isText: Flag to indicate of the file is text(TRUE) or binary (FALSE) file type.
    """
    fileUID: str = ''
    fmsFileTicket: str = ''
    fileName: str = ''
    isText: bool = False


@dataclass
class DateTimeRange(TcBaseObj):
    """
    A pair of DateTime objects representing the start and end date/time.
    
    :var startDate: The starting date/time value.
    :var endDate: Then ending date/time value.
    """
    startDate: datetime = None
    endDate: datetime = None


@dataclass
class FindFullTextSavedSearchesResponse(TcBaseObj):
    """
    Response from FindFullTextSavedSearches SOA
    
    :var mySavedSearches: My saved searches
    :var sharedSavedSearches: Shared saved searches
    :var serviceData: Service data
    """
    mySavedSearches: List[Awp0FullTextSavedSearch] = ()
    sharedSavedSearches: List[Awp0FullTextSavedSearch] = ()
    serviceData: ServiceData = None


class SearchFilterFieldSortType(Enum):
    """
    An enumeration to determine how fields used in filters need to be sorted. The fields can be sorted alphabetically
    in ascending order or in priority order from highest to lowest.
    """
    Alphabetical = 'Alphabetical'
    Priority = 'Priority'


class SearchFilterType(Enum):
    """
    An enumeration for different types of search filters like string, date, numeric.
    """
    StringFilter = 'StringFilter'
    DateFilter = 'DateFilter'
    NumericFilter = 'NumericFilter'


class SortDirection(Enum):
    """
    An enumeration indicating whether the sorting needs to be performed in ascending or descending order.
    """
    ASC = 'ASC'
    DESC = 'DESC'


"""
A map containing a list of search filters for each search filter field.
"""
SearchFilterMap = Dict[str, List[SearchFilter]]


"""
Map from Client Id to created FullTextSavedSearch object
"""
CreateFullTextSavedSearchOutput = Dict[str, Awp0FullTextSavedSearch]


"""
Map whoose key is the dataset UID and value is the vector of DatasetFileInfo objects.
"""
DatasetFileTicketInfosMap = Dict[str, List[DatasetFileInfo]]



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.ProjectManagement._2007_01.ScheduleManagement import MultipleScheduleLoadResponse, ScheduleChangeContainer
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def loadSchedule(cls, schedule: List[str]) -> MultipleScheduleLoadResponse:
        """
        Loads all the objects in a schedule needed for the scheduling logic.
        """
        return cls.execute_soa_method(
            method_name='loadSchedule',
            library='Internal-ProjectManagement',
            service_date='2007_01',
            service_name='ScheduleManagement',
            params={'schedule': schedule},
            response_cls=MultipleScheduleLoadResponse,
        )

    @classmethod
    def modifySchedule(cls, scheduleChangeContainer: ScheduleChangeContainer) -> ServiceData:
        """
        Updates all the affected scheduling objects based on the initial
        users request to the Application Interface.
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='modifySchedule',
            library='Internal-ProjectManagement',
            service_date='2007_01',
            service_name='ScheduleManagement',
            params={'scheduleChangeContainer': scheduleChangeContainer},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Core._2020_04.LogicalObject import AddMemAndPresentedPropsWriteInput
from tcsoa.gen.Internal.Core._2017_11.LogicalObject import AddMemberAndPresentedPropsResponse
from tcsoa.base import TcService


class LogicalObjectService(TcService):

    @classmethod
    def addMemAndPresentedPropsWithWrite(cls, addMembersAndPresentedProps: AddMemAndPresentedPropsWriteInput) -> AddMemberAndPresentedPropsResponse:
        """
        This operation adds a list of members and presented properties to an existing Logical Object Type based on the
        specified input with support to update the source property.  Presented properties are part of Logical Object
        Type which also composed d of root, and member objects and presented properties.  Logical Objects are used to
        represent logical views of complex data models in terms of a simplified structure of members and properties.
        
        This operation also adds presented properties from an existing Logical Object Type to the related Attribute
        Exchange Configuration object.
        
        Use cases:
        Use Case 1: 
        This operation is invoked to add members and presented properties to an existing Logical Object Type.
        
        Use Case 2: 
        This operation also adds presented properties from an existing Logical Object Type to the related Attribute
        Exchange Configuration object.
        """
        return cls.execute_soa_method(
            method_name='addMemAndPresentedPropsWithWrite',
            library='Internal-Core',
            service_date='2020_04',
            service_name='LogicalObject',
            params={'addMembersAndPresentedProps': addMembersAndPresentedProps},
            response_cls=AddMemberAndPresentedPropsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ProductImageInfo(TcBaseObj):
    """
    The input structure contains object(s) for which product image is to be set, the business object of the dataset 
    representing the image and the product Bill Of Processes (BOP) context.
    
    :var targetObjects: The list of business objects for which the product image is to be set.
    The valid types are as follows
    - Mfg0BvrProcessStation - process station.
    - Mfg0BvrProcessLine - process line.
    - Mfg0BvrProcessArea - process area.
    - Mfg0BvrPlantBOP - plant Bill Of Processes (BOP). 
    
    
    :var contextProductBOP: The business object representing the top line of the Product Bill Of Processes(BOP). This
    Product BOP is the context for which the product image is set with the targetObjects and is of type
    MEProductBOPRevision.
    This could be NULL provided that there is only one product BOP linked to the plant BOP. In case multiple product
    BOPs are linked then corresponding error will be reported.
    :var imageDataset: The business object of the dataset representing the product image. The same product image will
    be common for all the objects in the input parameter targetObjects.
    """
    targetObjects: List[BusinessObject] = ()
    contextProductBOP: BusinessObject = None
    imageDataset: BusinessObject = None


@dataclass
class TwpInfo(TcBaseObj):
    """
    Structure contains the object and its corresponding Time Way Plan (TWP) information.
    
    :var object: The object for which information is fetched.
    The valid types are 
    - Mfg0BvrProcessStation - process station
    - Mfg0BvrProcessLine - process line
    - Mfg0BvrProcessArea - process area
    - Mfg0BvrPlantBOP - plant bill of process (BOP) or
    - Mfg0BvrOperation - operation.
    
    
    :var detailedInfo: The map of string which is a data identifier and its corresponding Data.
    Description of valid strings and its data 
        "Operations": Represents the operations executed under the station. 
        "ExecutionPositions": Represents the execution positions for the object of TwpInfo stucture.
        "Direction": Represents the direction of the station in the plant.
        "ProductImage": Represents the product image for the station.
        "PlantCarpet": Represents the carpet diagram of the plant.
        "TWPLocationForms": Represents the business objects of the TWP Location form associated with the station.
    """
    object: BusinessObject = None
    detailedInfo: DetailedInformation = None


@dataclass
class TwpInfoInput(TcBaseObj):
    """
    The input structure contains object(s) for which information is required, the list of string specifying what
    information is required and the product Bill Of Process (BOP) context. The object can be process station(s),
    process line(s), process area(s), or plant BOP.
    
    :var requestedObjects: The list of business object for which TWP information is required. The valid types are
    process station, process line, process area or plant Bill Of Process (BOP). 
    :var contextProductBOP: The business object representing the top line of the Product Bill Of Process (BOP). This
    Product BOP is the context for which TWP information is fetched and is of type MEProductBOPRevision   .
    This could be NULL provided that there is only one product BOP linked to the plant BOP. In case multiple product
    BOPs are linked then corresponding error will be reported.
    :var requiredData: List of string specifying what information is required.
    The valid options are:
        OperationDetails: To fetch the operation details of given station.
    Response will consist of information about all the operations under the given station, the allocated time for those
    operations and their execution position. Other station related information such as length, width, X-Y Coordinates
    specifying its location in plant, orientation and direction will also be part of response.
        ExecutionPositions : To fetch the execution positions of the given station.
    Other station related information such as length, width, X-Y Coordinates specifying its location in plant,
    orientation and direction will also be part of response.
        ProductImage : To fetch the product image which will be displayed on the station in TWP view.
        PlantCarpet : To fetch the carpet image of the plant. This image is shown in the TWP view as plant layout.
    """
    requestedObjects: List[BusinessObject] = ()
    contextProductBOP: BusinessObject = None
    requiredData: List[str] = ()


@dataclass
class TwpResponse(TcBaseObj):
    """
    The response contains a structure of the TWP data and the ServiceData. The possible errors reported are:
        251048 - The Plant Bill Of Process (BOP) is linked to multiple Product BOPs. Please select one of the Product
    BOP as context.
        251049 - The input Product Bill Of Process (BOP) and Plant BOP are not linked.
    
    
    :var serviceData: The service data containing partial errors if any.
    :var infoMap: A map of business objects for which TWP information is requested and its corresponding TWP
    information.
    """
    serviceData: ServiceData = None
    infoMap: InformationMap = None


@dataclass
class Data(TcBaseObj):
    """
    The structure holds the property value.
    
    :var dataType: Type of the data. Valid types are "Boolean", "Character", "Integer", "Double", "String", "Tag" and
    "Date". One of the data in this structure need to be accessed based on the data type string.
    :var boolProperties: The list of Boolean values.
    :var charProperties: The string representing the list of characters. Each character in the string is a value of the
    property.
    :var integerProperties: The list of integer values.
    :var doubleProperties: The list of double values.
    :var stringProperties: The list of string values.
    :var tagProperties: The list of business objects.
    :var dateProperties: The list of dates.
    """
    dataType: str = ''
    boolProperties: List[bool] = ()
    charProperties: str = ''
    integerProperties: List[int] = ()
    doubleProperties: List[float] = ()
    stringProperties: List[str] = ()
    tagProperties: List[BusinessObject] = ()
    dateProperties: List[datetime] = ()


"""
The map of string which is a data identifier and its corresponding Data.
"""
DetailedInformation = Dict[str, Data]


"""
A map of business objects for which TWP information is requested and its corresponding TWP information.
"""
InformationMap = Dict[BusinessObject, TwpInfo]



from typing import Optional, Dict, List

from tcsoa.basics import TcSoaBasics
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Query.services import SavedQueryService
from tcsoa.gen.Query._2019_06.SavedQuery import BusinessObjectQueryInput3
from tcsoa.gen.Query._2010_09.SavedQuery import BusinessObjectQueryClause


class TcSearchBasics:
    _cached_queries: Dict[str, BusinessObject] = None

    @classmethod
    def get_query_by_name(cls, query_name) -> Optional[BusinessObject]:     # returns: ImanQuery
        if cls._cached_queries is None:
            cls._cached_queries = dict()
            saved_queries_response = SavedQueryService.getSavedQueries()
            model_objs = saved_queries_response.serviceData.modelObjects
            for query in saved_queries_response.queries:
                cls._cached_queries[query.name] = model_objs[query.query.uid]
        return cls._cached_queries.get(query_name, None)

    @classmethod
    def exec_query(cls, query: BusinessObject, args: Dict[str, any], limit: int = 0, apply_defaults=True) -> List[BusinessObject]:
        entries, values = [], []
        if args:
            for k, v in args.items():
                entries.append(k)
                values.append(v)

        if apply_defaults:
            query_desc_resp = SavedQueryService.describeSavedQueries([query])
            for field in query_desc_resp.fieldLists[0].fields:
                if field.entryName and field.value and field.entryName not in entries:
                    entries.append(field.entryName)
                    values.append(field.value)

        exec_result = SavedQueryService.executeSavedQuery(
            query=query,
            entries=entries,
            values=values,
            limit=limit,
        )
        return exec_result.objects

    @classmethod
    def search_bo(cls, bo_name: str, props: Dict[str, str], max_result_num: int = 1):
        response = SavedQueryService().executeBOQueriesWithSort([
            BusinessObjectQueryInput3(
                boTypeName=bo_name,
                maxNumToReturn=max_result_num,
                requestId='abc',
                clientId='abc',
                clauses=[
                    BusinessObjectQueryClause(
                        propName=prop_name,
                        propValue=prop_val,
                        mathOperator='=',
                        logicOperator='AND'
                    )
                    for prop_name, prop_val in props.items()
                ],
            )
        ])
        found_obj_uids = response.arrayOfResults[0].objectUIDS
        return list(TcSoaBasics.load_objects(found_obj_uids).values())



from __future__ import annotations

from tcsoa.gen.Internal.MultiSite._2007_06.ObjectDirectory import ItemIdRecordReturnStruct, PublicationRecordProperties
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class LocatePublishedObjectResponse(TcBaseObj):
    """
    'LocatePublishedObjectResponse' structure contains locating information for the input object and 'Servicedata'.
    
    :var owningSiteId: List of owning siteids for all the input Published Objects tag strings.
    
    :var serverCorrelID: Server correlation ID.
    :var userDataInput: Added for future use.
    :var serviceData: The Service Data with failure message if any.
    """
    owningSiteId: List[int] = ()
    serverCorrelID: str = ''
    userDataInput: str = ''
    serviceData: ServiceData = None


@dataclass
class ODSOperationResponse(TcBaseObj):
    """
    'ODSOperationResponse' structure contains ODS response information and 'ServiceData'.
    
    :var serverCorrelID: Server correlation ID.
    :var userDataOutput: Added for future use.
    :var serviceData: The Service Data with failure message if any.
    """
    serverCorrelID: str = ''
    userDataOutput: str = ''
    serviceData: ServiceData = None


@dataclass
class QueryItemIdRecordResponse(TcBaseObj):
    """
    'QueryItemIdRecordResponse' structure contains output query results in 'ItemIdRecordReturnStruct' object and
    'ServiceData'.
    
    :var queryResults: List of 'ItemIdRecordReturnStruct' object which contains owning site id and flag for
    registration 
    for each input 'BoMfkInfo' object.
    :var serverCorrelID: Server correlation ID.
    :var userDataOutput: Added for future use.
    :var serviceData: The Service Data with failure message if any.
    """
    queryResults: List[ItemIdRecordReturnStruct] = ()
    serverCorrelID: str = ''
    userDataOutput: str = ''
    serviceData: ServiceData = None


@dataclass
class QueryPublicationRecordsResponse(TcBaseObj):
    """
    'QueryPublicationRecordsResponse' structure contains information returned from the query and 'ServiceData'.
    
    :var publishedObjTagAsString: List of published objects tag strings which returned as remote search results.
    
    :var serverCorrelID: Server correlation ID.
    :var userDataOutput: Added for future use.
    :var serviceData: The Service Data with failure message if any.
    """
    publishedObjTagAsString: List[str] = ()
    serverCorrelID: str = ''
    userDataOutput: str = ''
    serviceData: ServiceData = None


@dataclass
class ClientInfoProperties(TcBaseObj):
    """
    'ClientInfoProperties' structure contains requesting ODS client information.
    
    :var clientSiteId: Requesting client siteid.
    :var targetServerSiteId: Target server siteid.
    :var userName: Logged in username of requesting client.
    :var osUserName: OS logged in username of requesting client node.
    
    :var groupName: Group name of userid of requesting client.
    
    :var roleName: Role name of userid of requesting client.
    
    :var clientCorrelID: Correlation Id of requesting client.
    :var userDataInput: Added for future use.
    :var customAttributeNames: String map of custom attributes of published Object and ODS custom attribute. This is
    used in case of extensible ODS feature.
    :var clientNode: Node name of requesting client.
    :var clientReleaseVersion: Teamcenter release version of requesting client.
    :var clientMinorReleaseVersion: Minor release version of requesting client.
    
    :var clientServiceReleaseVersion: Teamcenter release version of requesting client server.
    
    :var clientMaintPackVersion: Teamcenter maintaince pack version of requesting client.
    
    :var clientOdsMajorVersion: Client ODS major version.
    :var clientOdsMinorVersion: Client ODS minor version.
    :var userId: Logged in userid of requesting client.
    """
    clientSiteId: int = 0
    targetServerSiteId: int = 0
    userName: str = ''
    osUserName: str = ''
    groupName: str = ''
    roleName: str = ''
    clientCorrelID: str = ''
    userDataInput: str = ''
    customAttributeNames: MapDataLogger = None
    clientNode: str = ''
    clientReleaseVersion: int = 0
    clientMinorReleaseVersion: int = 0
    clientServiceReleaseVersion: int = 0
    clientMaintPackVersion: int = 0
    clientOdsMajorVersion: int = 0
    clientOdsMinorVersion: int = 0
    userId: str = ''


@dataclass
class DescribePublicationRecordsResponse(TcBaseObj):
    """
    'DescribePublicationRecordsResponse' structure object which has 'ServiceData' (contains failure with error message
    if any). It also contains vector of PublicationRecord properties.
    
    :var records: List of 'PublicationRecordProperties' object for all the input PublishedObjects tag strings.
    :var serverCorrelID: Server correlation ID.
    :var userDataOutput: Added for future use.
    :var serviceData: The 'ServiceData' with failure message if any.
    """
    records: List[PublicationRecordProperties] = ()
    serverCorrelID: str = ''
    userDataOutput: str = ''
    serviceData: ServiceData = None


"""
MapDataLogger
"""
MapDataLogger = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Internal.RevRuleMgmt._2019_12.RevisionRuleAdministration import CreateOrUpdateRevRuleResp, CreateUpdateRevRuleInput, RevRuleInfoResponse
from tcsoa.base import TcService


class RevisionRuleAdministrationService(TcService):

    @classmethod
    def getRevisionRuleInfo(cls, revRule: str) -> RevRuleInfoResponse:
        """
        Retreives description, rule entries and nested effectivity information for input RevisionRule.
        
        Use cases:
        User selects RevisionRule to view the details like rule entries so that user can make informed decision on
        configuring the content.
        
        Exceptions:
        >The operation can raise a  ServiceExeption may contain following errors.
        710001  An internal error has occurred in the Configuration Management module. Please report this error to your
        system administrator.
        710041  Revision Rule does not exist.
        """
        return cls.execute_soa_method(
            method_name='getRevisionRuleInfo',
            library='Internal-RevRuleMgmt',
            service_date='2019_12',
            service_name='RevisionRuleAdministration',
            params={'revRule': revRule},
            response_cls=RevRuleInfoResponse,
        )

    @classmethod
    def createOrUpdateRevisionRule(cls, inputData: CreateUpdateRevRuleInput) -> CreateOrUpdateRevRuleResp:
        """
        Creates or updates a RevisionRule based on input information.
        
        Use cases:
        Use Case 1: Creation of Revision Rule
        In Active Workspace, RevisionRule can be created from "Revision Rules" sub-location where all existing rules
        along with their clauses will be listed, given that user have privileges to create a RevisionRule.
        
        Use Case 2: Modification of existing Revision Rule:
        In Active Workspace, RevisionRules can be modified inside "Revision Rules" sub-location , given that user have
        privileges to modify a RevisionRule.Clauses of a RevisionRule can be re-ordered to change the precedence by
        moving the clause up/down using the corresponding controls.New clause from list of available clauses can be
        added or a clause can be removed from existing list of clauses.
        
        Use Case 3: Modification of existing Revision Rule inside ACE:
        Inside ACE (Active Content Experience), adhoc changes to the RevisionRule clauses can be performed to configure
        product structure. Addition of new clauses, removal of exisiting clauses or re-ordering of the clauses can be
        done to create a 'modified' version copy of the original RevisionRule which will be available to the current
        user only.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateRevisionRule',
            library='Internal-RevRuleMgmt',
            service_date='2019_12',
            service_name='RevisionRuleAdministration',
            params={'inputData': inputData},
            response_cls=CreateOrUpdateRevRuleResp,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, User
from tcsoa.gen.Administration._2006_03.IRM import GetProtectionReportResponse, PrivilegeSettingInput, GetACLInfoResponse, CheckAccessorPrivilegesResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class IRMService(TcService):

    @classmethod
    def removeAccessor(cls, objects: List[BusinessObject], accessorType: str, accessorId: str) -> ServiceData:
        """
        This operation removes the specified accessors from the given objects. Objects in the given list can be either
        AM_ACL (named Access Control List) objects or POM_application_object. If the object is a named ACL then the
        Access Control Entry (ACE)  from the named ACL is removed.  If the object is a POM_application_object then
        object ACL entry with the specified accessor type and accessor id is removed.  Objects on which given accessor
        is removed successfully are returned in the updated list of the ServiceData.  Objects on which accessor removal
        resulted in an error are returned in the list of failures in the ServiceData.  Invalid accessor type and
        accessor ID will result in error code 525120.
        
        Use cases:
        Modifying a Named ACL or removing an object ACL from Teamcenter Rich Application Client (RAC) calls this
        operation.
        """
        return cls.execute_soa_method(
            method_name='removeAccessor',
            library='Administration',
            service_date='2006_03',
            service_name='IRM',
            params={'objects': objects, 'accessorType': accessorType, 'accessorId': accessorId},
            response_cls=ServiceData,
        )

    @classmethod
    def setPrivileges(cls, privilegeSettings: List[PrivilegeSettingInput], accessorType: str, accessorId: str) -> ServiceData:
        """
        This operation can be used to grant or deny a set of privileges to the specified accessor type and accessor id
        on the given object.  Either a named Access Control List (ACL) or a POM_application_object can be submitted as
        input to this operation.  If the object is a POM_application_object then an object ACL will be added on the
        object. If the object is a named ACL then an ACE entry is either modified or added to the named ACL. For
        invalid Accessor type and Accessor Id this operation will return the error 525101.
        
        Use cases:
        - Updating named ACL objects by either modifying existing ACE entries or by adding new ACE entries for the
        specified accessor.
        - Adding new object ACLs on a POM_application_object for the specified accesor.
        
        """
        return cls.execute_soa_method(
            method_name='setPrivileges',
            library='Administration',
            service_date='2006_03',
            service_name='IRM',
            params={'privilegeSettings': privilegeSettings, 'accessorType': accessorType, 'accessorId': accessorId},
            response_cls=ServiceData,
        )

    @classmethod
    def getEffectiveACLInfo(cls, objects: List[BusinessObject]) -> GetACLInfoResponse:
        """
        This operation can be used to get the effective Access Control List (ACL) information.  Effective ACL
        information displays the Access Control Entries (ACEs) that are applicable to the business object with respect
        to the user for whom access privileges are being evaluated on the object. Applicable ACEs are picked up from
        the ACLs that are configured against the Access Manager Rules which are evaluated to true for the given object
        based on the object's type, class, attributes, status and project to which it is assigned to etc.  By looking
        at the effective ACL table end user will be able to understand what privileges or granted, what privileges are
        denied for the user on the object and, what ACL and Accessor Type are involved in either granting or denying a
        particular access privilege. This operation can be used to get all the information required to render the
        effective ACL table. Limitation with this operation is it does not support localization. Hence all the strings
        returned by this or internal values. For more information on ACLs, ACEs, effective ACLs please refer to the
        Access Manager guide in Teamcenter.
        
        Use cases:
        ACL list dialog in Teamcenter Rich Application Client (RAC) used to call this operation to get the effective
        ACL information before it was replaced with the new operation getEffectiveACLInfo2 that supports localization
        of ACL names, Accessor type names and privilege names.
        """
        return cls.execute_soa_method(
            method_name='getEffectiveACLInfo',
            library='Administration',
            service_date='2006_03',
            service_name='IRM',
            params={'objects': objects},
            response_cls=GetACLInfoResponse,
        )

    @classmethod
    def getExtraProtectionInfo(cls, user: User, objects: List[BusinessObject]) -> GetProtectionReportResponse:
        """
        This operation can be used to get the additional access protection information for a given user on a set of
        business objects.  Additional protection information can be used to understand what Named ACL, what Accessor
        and what AM rule path are involved in arriving at the verdict for an access privilege on a given object for the
        given user. This helps the user to understand why a particular privilege on the given object is granted or
        denied for the given user.  This operation does not support localization. Hence all the strings returned by
        this operation are internal values.  This operation is replaced by a new operation getExtraProtectionInfo2 that
        supports localization.
        
        Use cases:
        Extra Protection Information dialog in Teamcenter Rich Application Client (RAC) used to call this operation
        before it was replaced with the new operation getExtraProtectionInfo2 that supports localization.
        """
        return cls.execute_soa_method(
            method_name='getExtraProtectionInfo',
            library='Administration',
            service_date='2006_03',
            service_name='IRM',
            params={'user': user, 'objects': objects},
            response_cls=GetProtectionReportResponse,
        )

    @classmethod
    def checkAccessorsPrivileges(cls, groupMember: BusinessObject, objects: List[BusinessObject], privilegeNames: List[str]) -> CheckAccessorPrivilegesResponse:
        """
        This operation gets the verdicts for the given access privileges for the given GroupMember on the given set of
        business objects.  This operation finds the accessors for the combination of given groupMember's user, role and
        group and then uses the list of found accessors to determine the verdicts for the given access privileges for
        the given GroupMember on the given business objects. The business objects can be any POM_object.  Privilege
        Names must be internal names of the Access Manager AM_Privilege objects.  If a privilege object with any of the
        given names does not exist in the system then this operation will return the error 525101.  However, evaluation
        of the valid privilege names will continue. Following are the list of valid privilege names. For functional
        description about these privileges please refer to the Access Manager guide in Teamcenter documentation.
        
        ADD_CONTENT
        ASSIGN_TO_PROJECT
        Administer_ADA_Licenses
        BATCH_PRINT
        CHANGE
        CHANGE_OWNER
        CICO
        COPY
        DELETE
        DEMOTE
        DIGITAL_SIGN
        EXPORT
        IMPORT
        IP_ADMIN
        IP_Classifier
        ITAR_ADMIN
        ITAR_Classifier
        MARKUP
        PROMOTE
        PUBLISH
        READ
        REMOTE_CICO
        REMOVE_CONTENT
        REMOVE_FROM_PROJECT
        SUBSCRIBE
        TRANSFER_IN
        TRANSFER_OUT
        TRANSLATION
        UNMANAGE
        WRITE
        WRITE_ICOS
        """
        return cls.execute_soa_method(
            method_name='checkAccessorsPrivileges',
            library='Administration',
            service_date='2006_03',
            service_name='IRM',
            params={'groupMember': groupMember, 'objects': objects, 'privilegeNames': privilegeNames},
            response_cls=CheckAccessorPrivilegesResponse,
        )



import enum
from typing import List


class Severity(enum.Enum):
    information = 1
    warning = 2
    error = 3
    user_error = 4


class TcError(Exception):
    def __init__(self, message: str, code: int, level: Severity, inner_exceptions: List['InnerException'] = ()):
        self.message = message
        self.code = code
        self.level = level
        self.inner_exceptions = inner_exceptions
        super(TcError, self).__init__()

    def __str__(self):
        return f"[{self.level.name}] Code {self.code} - {self.message}"


class ServiceException(TcError):
    pass


class InvalidCredentialsException(TcError):
    pass


class InternalServerException(TcError):
    pass


class InnerException(TcError):
    pass



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.ActiveWorkspaceBom._2015_10.OccurrenceManagement import OccurrenceSelection
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class OccurrenceManagementService(TcService):

    @classmethod
    def saveSelections(cls, selections: List[OccurrenceSelection]) -> ServiceData:
        """
        This operation is used to save all the occurrences selected by the user in the Active Content Experience module
        of the Active Workspace Client. The selection data is stored in the Awb0BookmarkSubsetData object related to
        the Awb0BookmarkProductData object.
        
        Use cases:
        This service is intended to be called from within the Active Content Experience (ACE) module of the Active
        Workspace Client (AW). The following is an example scenario on how this service could be used.
        - User selects one or more occurrences in the ACE module of AW. 
        - User then clicks on an action command, such as the "Open in Solid Edge". 
        - The Open in Solid Edge command will first invoke this saveSelection() operation to save the selected
        occurrences to Teamcenter. 
        - The Open in Solid Edge command will then launch the Product Context object to the Solid Edge CAD integration. 
        - The Solid Edge CAD integration would then call the createBOMWindowsFromProductContexts() SOA operation to
        create a BOM Window based on the Product Context object launched from AW. 
        - The createBOMWindowsFromProductContexts() operation would return the BOM Window created for the Product
        Context object and the list of BOMLine objects corresponding to the occurrences selected in AW.
        
        """
        return cls.execute_soa_method(
            method_name='saveSelections',
            library='Internal-ActiveWorkspaceBom',
            service_date='2015_10',
            service_name='OccurrenceManagement',
            params={'selections': selections},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Internal.MultiSite._2011_06.ObjectDirectory import BoMfkInfo, ItemIdRecordsProperties, PublicationRecordProperties
from tcsoa.gen.Internal.MultiSite._2012_02.ObjectDirectory import ODSOperationResponse, ClientInfoProperties, QueryItemIdRecordResponse, LocatePublishedObjectResponse, DescribePublicationRecordsResponse, QueryPublicationRecordsResponse
from tcsoa.gen.Internal.MultiSite._2012_02.RemoteOperation import IDSM11EndImportIn, IDSM11StartExportItemIdListRes, IDSM11VersionCheckIn, IDSM11VerifyObjectsRes, IDSM11DescribeItemIn, IDSM11StartAskInfoIn, IDSM11PerformSendObjectsRes, IDSM11GetErrorStackRes, IDSM11SetSyncOptionsIn, IDSM11DistributedAppRes, IDSM11DescribeObjectIn, IDSM11CheckSyncStatusIn, IDSM11DescribeObjectRes, IDSM11RemoteNotificationRes, IDSM11StartImportRes, IDSM11EndAskInfoIn, IDSM11FailureRes, IDSM11CheckSyncStatusRes, IDSM11SetSyncOptionsRes, IDSM11GetErrorStackIn, IDSM11DescribeItemRes, IDSM11StartExportIn, IDSM11ExportStatusIn, IDSM11StartExportItemIdListIn, IDSM11VerifyObjectsIn, IDSM11EndExportItemIdListIn, IDSM11RemoteNotificationIn, IDSM11VersionCheckRes, IDSM11RepublishObjectIn, IDSM11StartSendObjectsIn, IDSM11StartExportRes, IDSM11EndSendObjectsIn, IDSM11LocateObjectRes, IDSM11XferInfoFileRes, IDSM11StartSendObjectsRes, IDSM11ExportStatusRes, IDSM11DistributedAppIn, IDSM11StartImportIn, IDSM11StartAskInfoRes, IDSM11XferInfoFileIn, IDSM11EndImportRes, IDSM11LocateObjectIn, IDSM11PerformSendObjectsIn
from typing import List
from tcsoa.base import TcService


class RemoteOperationService(TcService):

    @classmethod
    def idsm11CheckSyncStatusSvc(cls, inputs: IDSM11CheckSyncStatusIn) -> IDSM11CheckSyncStatusRes:
        """
        The 'idsm11CheckSyncStatusSvc' operation is used to check the sync status of the input set of object(s) in the
        'IDSM11CheckSyncStatusIn' structure. 
        
        This operation is integrated with MultiSite Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based MultiSite environment. This operation acts as a wrapper call to server
        side RPC function and hence is invoked from within the MultiSite framework. Consequently no other clients can
        invoke this operation.
        
        The corresponding RPC call is idsm11_check_sync_status_1_svc().
        
        
        Use cases:
        - This operation is called from the auto sync handler to retrieve the object sync state. The auto sync handler
        used to extend and customize task actions. 
        
        """
        return cls.execute_soa_method(
            method_name='idsm11CheckSyncStatusSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11CheckSyncStatusRes,
        )

    @classmethod
    def idsm11DescribeItemSvc(cls, inputs: IDSM11DescribeItemIn) -> IDSM11DescribeItemRes:
        """
        The 'idsm11DescribeItemSvc' operation retrieves item information for a given item.
        
        This operation is integrated with MultiSite Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based MultiSite environment. This operation acts as a wrapper call to server
        side RPC function and hence is invoked from within the MultiSite framework. Consequently no other clients can
        invoke this operation.
        
        The corresponding RPC call is 'idsm11_describe_item_1_svc()'.
        
        
        Use cases:
        - This operation is called from data_share utility, for reporting the duplicate items. For the supplied item
        id, this operation will get item information available with the remote Integrated Distributed Services Manager
        (IDSM) server and reports the duplicate items. 
        
        """
        return cls.execute_soa_method(
            method_name='idsm11DescribeItemSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11DescribeItemRes,
        )

    @classmethod
    def idsm11DescribeObjectSvc(cls, inputs: IDSM11DescribeObjectIn) -> IDSM11DescribeObjectRes:
        """
        The 'idsm11DescribeObjectSvc' operation used to retrieve the object description from Integrated Distributed
        Services Manager (IDSM) server.  Consumer should provide the client information and the tag of the object, as
        an input.
        
        This operation is integrated with Multi-Site  Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site  environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site  framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding rpc call is 'idsm11_describe_object_1_svc().'
        
        
        Use cases:
        - To import the remote objects at the local site, object(s) published to the Object Directory Service (ODS)
        needs to be first searched and then imported. While loading the ODS published remote objects, this operation is
        called to get the object information.  
        
        """
        return cls.execute_soa_method(
            method_name='idsm11DescribeObjectSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11DescribeObjectRes,
        )

    @classmethod
    def idsm11DistributedAppSvc(cls, inputs: IDSM11DistributedAppIn) -> IDSM11DistributedAppRes:
        """
        This distributed call checks, if the Integrated Distributed Services Manager (IDSM) provides the supplied
        service to the client. The op codes are used to specify the operation. Please refer the
        'IDSM11DistributedAppIn' structure for the list of op codes.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is 'idsm11_distributed_app_1_svc().'
        
        
        Use cases:
        - This operation is called from the DIST_IDSM_commit_synchronous_remote_import(), to check if the commit
        synchronous remote import operation supported by IDSM server.
        
        """
        return cls.execute_soa_method(
            method_name='idsm11DistributedAppSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11DistributedAppRes,
        )

    @classmethod
    def idsm11EndAskInfoSvc(cls, inputs: IDSM11EndAskInfoIn) -> IDSM11FailureRes:
        """
        The 'idsm11EndAskInfoSvc' operation is used to finish the info file transfer.  This is the last operation in
        the sequence of transferring info file and finishes the info file transfer process.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is 'idsm11_end_ask_info_1_svc().'
        
        
        Use cases:
        This operation is called while creating or updating the dataset. 'idsm11StartAskInfoSvc' call will be made to
        get the schema info of the given site, then type and class-info will be fetched out of it, on successful
        completion of the call, info file is transferred to server staging directory using 'idsm11XferInfoFileSvc'. The
        'idsm11EndAskInfoSvc' closes the communication channel and finishes the file transfer.
        """
        return cls.execute_soa_method(
            method_name='idsm11EndAskInfoSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11FailureRes,
        )

    @classmethod
    def idsm11EndExportItemIdListSvc(cls, inputs: IDSM11EndExportItemIdListIn) -> IDSM11FailureRes:
        """
        The 'idsm11EndExportItemIdListSvc' is used to delete the server staging directory, and set finished status. 
        This is the last operation in the sequence of remote item id import operation and finishes the remote item id
        import process.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding rpc call is' idsm11_end_export_item_id_list_1_svc().'
        
        
        Use cases:
        At the moment this API is not used, and it has been added considering future requirement.
        """
        return cls.execute_soa_method(
            method_name='idsm11EndExportItemIdListSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11FailureRes,
        )

    @classmethod
    def idsm11EndImportSvc(cls, inputs: IDSM11EndImportIn) -> IDSM11EndImportRes:
        """
        The 'idsm11EndImportSvc' operation imports remote business objects to the local site. When a user at Source
        Site selects business objects (e.g, Item) for remote export, these objects are written into object.meta file at
        the Source Site and transferred over to the Target Site. Integrated Distributed Services Manager (IDSM) service
        at the Target Site imports the content of this file and 'idsm11EndImportSvc' is the last operation in the
        sequence and cannot be called in isolation.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is 'idsm11_end_import_1_svc().'
        
        
        
        Use cases:
        To Remote Export business object(s), a series of operations needs to be performed and 'Idsm11EndImportSvc' gets
        invoked to complete the remote import transaction. 
        Operation sequence for remote export
        - idsm11StartImportSvc
        - idsm4XferImportFileSvc
        - idsm11EndImportSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm11EndImportSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11EndImportRes,
        )

    @classmethod
    def idsm11EndSendObjectsSvc(cls, inputs: IDSM11EndSendObjectsIn) -> IDSM11FailureRes:
        """
        The 'idsm11EndSendObjectsSvc' ends the system object s remote import process. The operation gets called, if
        abort flag is set to true by the previous operation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        The corresponding RPC call is' idsm11_end_ send_objects_1_svc().'
        
        
        Use cases:
        This operation is called from the distributed_execute utility. The utility is used to generate the 'item'
        reports locally as well as remotely. To generate the report, internally this utility calls the item_report
        utility.
        """
        return cls.execute_soa_method(
            method_name='idsm11EndSendObjectsSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11FailureRes,
        )

    @classmethod
    def idsm11ExportStatusSvc(cls, inputs: IDSM11ExportStatusIn) -> IDSM11ExportStatusRes:
        """
        The 'idsm11ExportStatusSvc' operation returns the export/import status of business objects.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding RPC call is 'idsm11_export_status_1_svc().'
        
        
        Use cases:
        - In case of the remote import, this operation is used to check the status of the remote import invoked by the
        earlier call 'idsm11StartExportSvc'. The status informs the remote site whether import is complete and if it s
        in process, then how many workspace objects are exported so far. The reported workspace objects will be in the
        multiples of 'checkClientAfterNWsos' supplied in the 'IDSM11ExportStatusIn' structure.
        - In case of the remote export, this operation is used to check the status of the remote export finished by the
        earlier call 'idsm11EndImportSvc'. The status informs the remote site whether export is complete and if it s in
        process, then how many workspace objects are exported so far. The reported workspace objects will be in the
        multiples of 'checkClientAfterNWsos' supplied in the 'IDSM11ExportStatusIn' structure.
        
        """
        return cls.execute_soa_method(
            method_name='idsm11ExportStatusSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11ExportStatusRes,
        )

    @classmethod
    def idsm11GetErrorStackSvc(cls, inputs: IDSM11GetErrorStackIn) -> IDSM11GetErrorStackRes:
        """
        The 'idsm11GetErrorStackSvc' operation is used to retrieve the error stack.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in
        Hypertext Transfer Protocol (HTTP) based Multi-Site environment. This operation acts as a wrapper call to
        server side RPC function and hence is invoked from within the Multi-Site framework. Consequently no other
        clients can invoke this operation.
        
        The corresponding rpc call is 'idsm11_get_error_stack_1_svc().'
        
        
        Use cases:
        While performing the checkpoint transaction, this operation is called from the data_share & data_sync utility.
        """
        return cls.execute_soa_method(
            method_name='idsm11GetErrorStackSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11GetErrorStackRes,
        )

    @classmethod
    def idsm11LocateObjectSvc(cls, inputs: IDSM11LocateObjectIn) -> IDSM11LocateObjectRes:
        """
        The 'idsm11LocateObjectSvc' operation locates the owning site of the remote object given in
        'IDSM11LocateObjectIn' input structure.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'idsm11_locate_object_1_svc().'
        
        
        Use cases:
        'idsm11LocateObjectSvc' operation invoked during remote import of stub object. Export Assembly from site1 to
        site2 with including BOM. On site2, open assembly in PSE (Product Structure Editor) and it shows components as
        Remote Object(s), do import on that object(s), which tries to find the owning site of the object. It tries to
        locate the stub object by finding its owning site by checking publication record for that object by calling
        'locatePublishedObjects'.
        """
        return cls.execute_soa_method(
            method_name='idsm11LocateObjectSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11LocateObjectRes,
        )

    @classmethod
    def idsm11PerformSendObjectsSvc(cls, inputs: IDSM11PerformSendObjectsIn) -> IDSM11PerformSendObjectsRes:
        """
        The 'idsm11PerformSendObjectsSvc' operation sends system objects to the remote site based on
        'IDSM11PerformSendObjectsIn' input.  This is the third operation in the sequence used in sending system objects
        such as Users, Groups, Persons, etc to the remote site. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'idsm11_perform_send_objects_1_svc().'
        
        
        Use cases:
        - idsm11PerformSendObjectsSvc operation gets invoked from   dsa_util  utility while sending system objects such
        as Users, Groups, Persons, etc to remote site.
        
        
        Operation sequence is as below
        - idsm11StartSendObjectsSvc
        - file transfer via DIST call
        - idsm11PerformSendObjectsSvc  
        - idsm11EndSendObjectsSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm11PerformSendObjectsSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11PerformSendObjectsRes,
        )

    @classmethod
    def idsm11RemoteNotificationSvc(cls, inputs: IDSM11RemoteNotificationIn) -> IDSM11RemoteNotificationRes:
        """
        The 'idsm11RemoteNotificationSvc' operation performs remote notification and subscription for object at remote
        site based on 'IDSM11RemoteNotificationIn' input structure.
        
        This is the last operation in the sequence and cannot be called in isolation. 
        
        This operation is integrated with Multi-Site RPC framework and is callable only in an http based 
         Multi-Site environment. This operation acts as a wrapper call to server side RPC function and 
        hence is invoked from within the Multi-Site framework. Consequently no other clients can 
        invoke this operation.
        
        Corresponding server side RPC call is 'idsm11_remote_notification_1_svc().'
        
        
        Use cases:
        'idsm11RemoteNotificationSvc' operation is called in automatic synchronization to notify and subscribe remote
        site in HTTP multisite environment.
        """
        return cls.execute_soa_method(
            method_name='idsm11RemoteNotificationSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11RemoteNotificationRes,
        )

    @classmethod
    def idsm11RepublishObjectSvc(cls, inputs: IDSM11RepublishObjectIn) -> IDSM11FailureRes:
        """
        The 'idsm11RepublishObjectSvc' operation updates the 'PublicationRecord' at the Object Directory Service (ODS)
        site for the object given in 'IDSM11RepublishObjectIn'.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        The corresponding RPC call is 'idsm11_republish_object_1_svc ().'
        
        
        Use cases:
        'idsm11RepublishObjectSvc' operation gets invoked for re-publishing the given object to ODS site after
        ownership transfer in use cases like remote import/export.  The ownership transfer operation performed on the
        business objects (BOs) change its ownership. After transfer of ownership, if it is not re-published then the BO
        has the wrong entry for the owning siteId in PublicationRecord at ODS site. In order to correct it, the BOs
        needs to be republished to the ODS server.
        """
        return cls.execute_soa_method(
            method_name='idsm11RepublishObjectSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11FailureRes,
        )

    @classmethod
    def idsm11SetSyncOptionsSvc(cls, inputs: IDSM11SetSyncOptionsIn) -> IDSM11SetSyncOptionsRes:
        """
        The 'idsm11SetSyncOptionsSvc' operation used to sets sync options on the objects after object import from
        importing site.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is' idsm11_set_sync_options_1_svc ().'
        
        
        Use cases:
        'idsm11SetSyncOptionsSvc' operation is called during remote import while syncing object(s) and is called from
        importing site in HTTP multisite environment.
        """
        return cls.execute_soa_method(
            method_name='idsm11SetSyncOptionsSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11SetSyncOptionsRes,
        )

    @classmethod
    def idsm11StartAskInfoSvc(cls, inputs: IDSM11StartAskInfoIn) -> IDSM11StartAskInfoRes:
        """
        The 'idsm11StartAskInfoSvc' operation is call used to get the information about the server side staging
        directory i.e. where the files are being transferred on the server side while get type-class mapping file from
        remote site. This basically opens a file transfer channel used to transfer files from the server.
        
        This is the first operation in the sequence which gets information about the Multi-Site staging directory and
        cannot be called in isolation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'idsm11_start_ask_info_1_svc().'
        
        
        Use cases:
        'idsm11StartAskInfoSvc' operation is invoked while getting type class files from the remote site in HTTP
        multisite environment. This is called from database_verify() utility.
        Operation sequence is as below:
        - idsm11StartAskInfoSvc
        - idsm11XferInfoFileSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm11StartAskInfoSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11StartAskInfoRes,
        )

    @classmethod
    def idsm11StartExportItemIdListSvc(cls, inputs: IDSM11StartExportItemIdListIn) -> IDSM11StartExportItemIdListRes:
        """
        The 'idsm11StartExportItemIdListSvc' operation gets information about the server side staging directory and
        output export files. 
        
        This is the first operation in the sequence and cannot be called in isolation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
                       
        Corresponding RPC call is' idsm11_start_export_item_id_list_1_svc().'
        
        
        Use cases:
        'idsm11StartExportItemIdListSvc' operation is called from data_share utility while finding duplicate itemId(s)
        at the remote site.
        """
        return cls.execute_soa_method(
            method_name='idsm11StartExportItemIdListSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11StartExportItemIdListRes,
        )

    @classmethod
    def idsm11StartExportSvc(cls, inputs: IDSM11StartExportIn) -> IDSM11StartExportRes:
        """
        The 'idsm11StartExportSvc' operation exports business objects at the remote site during a Remote Import
        transaction. When a user at Source Site selects business objects (e.g, Item) for remote import, the request is
        sent to the Target Site to export these objects into object.meta file at the Target Site and transferred over
        to the Source Site (requesting site). IDSM service at the Target Site exports the objects and creates
        object.meta file. The 'idsm11StartExportSvc' is the first operation in the sequence and hence cannot be called
        in isolation.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is' idsm11_start_export_1_svc().'
        
        
        Use cases:
        To remote import the business object, first the source site user creates an Item and publishes it to the
        default Object Directory Service (ODS) site.  The target site user performs remote search with the item id and
        imports it to the target site using the remote import operation. On successful completion of remote import
        operation, the item is imported to the target site database. To Remote Import the business object, the series
        of operations needs to be performed and this operation gets invoked as a first step in the Remote Import
        transaction. 
        The sequence of operations are as below 
        - idsm11StartExportSvc
        - idsm11ExportStatusSvc
        - idsm4XferExportFileSvc
        - idsm1EndExportSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm11StartExportSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11StartExportRes,
        )

    @classmethod
    def idsm11StartImportSvc(cls, inputs: IDSM11StartImportIn) -> IDSM11StartImportRes:
        """
        The 'idsm11StartImportSvc' operation gets information about the server side staging directory i.e. where the
        files are being transferred on the server during export.
        
        This is the first operation in the sequence and cannot be called in isolation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
                       
        Corresponding RPC call is 'idsm11_start_import_1_svc().'
        
        
        Use cases:
        To Remote Export business object(s), a series of operations needs to be performed and Idsm3StartImportSvc gets
        invoked as a first step. 
        Operation sequence for remote export.
        
        - idsm11StartImportSvc
        - idsm4XferImportFileSvc
        - idsm11EndImportSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm11StartImportSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11StartImportRes,
        )

    @classmethod
    def idsm11StartSendObjectsSvc(cls, inputs: IDSM11StartSendObjectsIn) -> IDSM11StartSendObjectsRes:
        """
        The 'idsm11StartSendObjectsSvc' operation gets information about the server side staging directory i.e. where
        the files are being transferred on the server side while sending system objects to the remote site. 
        
        This is the first operation in the sequence which gets information about the Multi-Site staging directory and
        cannot be called in isolation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is' idsm11_start_send_objects_1_svc().'
        
        
        Use cases:
        'idsm11StartSendObjectsSvc' operation gets invoked from   dsa_util  utility while sending system objects such
        as Users, Groups, Persons, etc to the remote site.
        Operation sequence is as below
        - idsm11StartSendObjectsSvc
        - file transfer via DIST call
        - idsm11PerformSendObjectsSvc  
        - idsm11EndSendObjectsSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm11StartSendObjectsSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11StartSendObjectsRes,
        )

    @classmethod
    def idsm11VerifyObjectsSvc(cls, inputs: IDSM11VerifyObjectsIn) -> IDSM11VerifyObjectsRes:
        """
        The 'idsm11VerifyObjectsSvc' operation is used to verify if the object exist at the target site given in
        'IDSM11VerifyObjectsIn' input structure.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
                       
        Corresponding RPC call is' idsm11_verify_objects_1_svc().'
        
        
        Use cases:
        'idsm11VerifyObjectsSvc' operation is called in case of importing stub objects from structure manager where it
        tries to check whether these objects exist at target site and their verdict in HTTP multisite environment.
        """
        return cls.execute_soa_method(
            method_name='idsm11VerifyObjectsSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11VerifyObjectsRes,
        )

    @classmethod
    def idsm11VersionCheckSvc(cls, inputs: IDSM11VersionCheckIn) -> IDSM11VersionCheckRes:
        """
        The 'idsm11VersionCheckSvc' operation gets the Integrated Distributed Services Manager (IDSM) server version.
        
        This operation is integrated with Multi-Site RPC framework and is callable only in HTTP based Multi-Site
        environment. This operation acts as a wrapper call to server side RPC function and hence is invoked from within
        the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        There is no RPC call added for this. But it can be added as per future requirement.
        
        
        Use cases:
        'idsm11VersionCheckSvc' operation is called prior to all IDSM requests made by client to check version of IDSM
        for compatibility in HTTP multisite environment.  It is also used by some of the Distributed (DIST) ITKs prior
        to make any IDSM request.
        """
        return cls.execute_soa_method(
            method_name='idsm11VersionCheckSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11VersionCheckRes,
        )

    @classmethod
    def idsm11XferInfoFileSvc(cls, inputs: IDSM11XferInfoFileIn) -> IDSM11XferInfoFileRes:
        """
        The 'idsm11XferInfoFileSvc' operation transfers the type class mapping file from the remote site(s) Multi-Site
        staging directory to the local site Multi-Site staging directory. 
        
        This is the second operation in the sequence and cannot be called in isolation. 
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'idsm11_xfer_info_file_1_svc().'
        
        
        Use cases:
        idsm11XferInfoFileSvc operation is invoked while getting type class files from the remote site in HTTP
        multisite environment. This is called from database_verify() utility.
        Operation sequence is as below:
        - idsm11StartAskInfoSvc
        - idsm11XferInfoFileSvc
        
        """
        return cls.execute_soa_method(
            method_name='idsm11XferInfoFileSvc',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='RemoteOperation',
            params={'inputs': inputs},
            response_cls=IDSM11XferInfoFileRes,
        )


class ObjectDirectoryService(TcService):

    @classmethod
    def locatePublishedObjects(cls, clientInfo: ClientInfoProperties, objTagAsString: List[str]) -> LocatePublishedObjectResponse:
        """
        The 'locatePublishedObjects' operation locates the published objects from the Object Directory Service (ODS)
        site for each 'objTagAsString' input given.  
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods11_locate_object_1_svc().'
        
        Use cases:
        'locatePublishedObjects' operation gets called while importing stub objects from Structure Manager. It tries to
        locate the stub object by finding its owning site by checking publication record for that object. Export
        Assembly from site1 to site2 with including BOM. On site2, open assembly in Structure Manager and it shows
        components as Remote Object(s), do import on that object(s), which tries to find the owning site of the object.
        """
        return cls.execute_soa_method(
            method_name='locatePublishedObjects',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'objTagAsString': objTagAsString},
            response_cls=LocatePublishedObjectResponse,
        )

    @classmethod
    def publishObjects(cls, clientInfo: ClientInfoProperties, records: List[PublicationRecordProperties]) -> ODSOperationResponse:
        """
        The 'publishObjects' operation creates the PublicationRecords at the Object Directory Service (ODS) site for
        each 'PublicationRecordProperties' input given.  
         
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods11_1_publish_object_1_svc().'
        
        
        Use cases:
        'publishObjects' operation gets called in ODS publish object operation. The business object(s) can be shared
        within the Multi-Site federation. To remote import (Pull case) it must be published to the ODS. It can either
        be called from Rich Client (RAC) or from command line data_share (publish switch) utility.
        """
        return cls.execute_soa_method(
            method_name='publishObjects',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'records': records},
            response_cls=ODSOperationResponse,
        )

    @classmethod
    def queryItemIdRecords(cls, clientInfo: ClientInfoProperties, mfkInfo: List[BoMfkInfo]) -> QueryItemIdRecordResponse:
        """
        The 'queryItemIdRecords' operation queries the ItemId Records from the central itemid registry for each
        'BoMfkInfo' input given. It checks whether input itemids are registered or not.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation
        
        Corresponding RPC call is 'ods11_1_describe_item_1_svc().'
        
        
        Use cases:
        'queryItemIdRecords' operation gets invoked in below use cases check whether given itemid is registered or not.
        - New Item creation when auto register is ON.
        - Registering  Item to  registry.
        
        """
        return cls.execute_soa_method(
            method_name='queryItemIdRecords',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'mfkInfo': mfkInfo},
            response_cls=QueryItemIdRecordResponse,
        )

    @classmethod
    def queryPublicationRecords(cls, clientInfo: ClientInfoProperties, clientLocale: str, queryString: str, sortKeys: List[str], sortOrder: List[int]) -> QueryPublicationRecordsResponse:
        """
        The 'queryPublicationRecords' operation queries the PublicationRecords from the Object Directory Service (ODS)
        site based on the input query String. This query string should be in POM SQL query format.
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods11_execute_query_1_svc().'
        
        
        Use cases:
        'queryPublicationRecords' operation is called in ODS Remote search operation from the Rich Client (RAC). It
        searches the ODS server for published objects that satisfy the search criteria.
        """
        return cls.execute_soa_method(
            method_name='queryPublicationRecords',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'clientLocale': clientLocale, 'queryString': queryString, 'sortKeys': sortKeys, 'sortOrder': sortOrder},
            response_cls=QueryPublicationRecordsResponse,
        )

    @classmethod
    def unpublishObjects(cls, clientInfo: ClientInfoProperties, objTagAsString: List[str]) -> ODSOperationResponse:
        """
        The 'unpublishObjects' operation deletes the PublicationRecord from the Object Directory Service (ODS) site for
        each 'objTagAsString' input given.  
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods11_unpublish_object_1_svc().'
        
        
        Use cases:
        'unpublishObjects' operation gets called in ODS unpublish object operation. In Multi-Site federation to remote
        import (Pull case) the Business Object (BO), it must be published to the ODS. And to revoke the sharing, the
        business object should be unpublished from ODS. It can either be called from Rich Client (RAC) or from command
        line data_share (unpublish switch) utility.
        """
        return cls.execute_soa_method(
            method_name='unpublishObjects',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'objTagAsString': objTagAsString},
            response_cls=ODSOperationResponse,
        )

    @classmethod
    def updatePublicationRecords(cls, clientInfo: ClientInfoProperties, records: List[PublicationRecordProperties]) -> ODSOperationResponse:
        """
        The 'updatePublicationRecords' operation updates the PublicationRecord at the Object Directory Service (ODS)
        site for each 'PublicationRecordProperties' input given.  
        
        This operation is integrated with Multi-Site Remote Procedure Call (RPC) framework and is callable only in HTTP
        based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence is
        invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods11_1_refresh_object_1_svc().'
        
        
        Use cases:
        'updatePublicationRecords' operation gets invoked in Integrates Distributed Service Manager (IDSM) re-publish
        object operation after object is exported to other site with ownership transfer in use cases like remote
        import/export. The ownership transfer operation performed on the business objects (BOs) change its ownership.
        After transfer of ownership, if it is not re-published then the BO has the wrong entry for the owning siteId in
        PublicationRecord at ODS site. In order to correct it, the BOs needs to be republished to the ODS server.
        """
        return cls.execute_soa_method(
            method_name='updatePublicationRecords',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'records': records},
            response_cls=ODSOperationResponse,
        )

    @classmethod
    def createItemIdRecords(cls, clientInfo: ClientInfoProperties, records: List[ItemIdRecordsProperties]) -> ODSOperationResponse:
        """
        The 'createItemIdRecords' operation creates Item Records in the item registry based on
        'ItemIdRecordsProperties' input.
        
        This operation is integrated with Multi-Site Remote Procedure Call ( RPC) framework and is callable only in an
        HTTP based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence
        is invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods11_1_register_item_1_svc()'.
        
        Use cases:
        - Admin user(s) set the ITEM_id_always_register_on_creation to TRUE on all sites within a Multi-Site
        federation. On item creation the items created automatically registers with the central item registry.
        - Admin user(s) can register existing item(s) with the central item registry, using Register Item Id command in
        the Rich Client(RAC) or command line utility data_share f=register.
        
        """
        return cls.execute_soa_method(
            method_name='createItemIdRecords',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'records': records},
            response_cls=ODSOperationResponse,
        )

    @classmethod
    def deleteItemIdRecords(cls, clientInfo: ClientInfoProperties, mfkInfo: List[BoMfkInfo]) -> ODSOperationResponse:
        """
        The 'deleteItemIdRecords' operation deletes Item Records from the item registry for each 'BoMfkInfo' input.
        
        This operation is integrated with Multi-Site Remote Procedure Call ( RPC) framework and is callable only in an
        HTTP based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence
        is invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        Corresponding RPC call is 'ods11_1_unregister_item_1_svc().'
        
        
        Use cases:
        Admin user(s) can unregister existing item(s) from the central item registry, using Unregister Item Id command
        in the Rich Client (RAC) or command line utility data_share  f=unregister.
        """
        return cls.execute_soa_method(
            method_name='deleteItemIdRecords',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'mfkInfo': mfkInfo},
            response_cls=ODSOperationResponse,
        )

    @classmethod
    def describePublicationRecords(cls, clientInfo: ClientInfoProperties, publishedObjTagsAsString: List[str], clientLocale: str) -> DescribePublicationRecordsResponse:
        """
        The 'describePublicationRecords' operation returns the 'PublicationRecord' properties for a given list of
        published objects. 
         
        This operation is integrated with Multi-Site Remote Procedure Call ( RPC) framework and is callable only in an
        HTTP based Multi-Site environment. This operation acts as a wrapper call to server side RPC function and hence
        is invoked from within the Multi-Site framework. Consequently no other clients can invoke this operation.
        
        corresponding RPC call is 'ods11_describe_object_1_svc().'
        
        
        Use cases:
        'describePublicationRecords' operation gets called via Object Directory Service (ODS) remote search in HTTP
        Multi-Site environment.  ODS Remote search returns list of the Published objects as per search criteria. For
        the list of published object, 'describePublicationRecords' operation is called to get the publicationRecord
        properties on each object.
        """
        return cls.execute_soa_method(
            method_name='describePublicationRecords',
            library='Internal-MultiSite',
            service_date='2012_02',
            service_name='ObjectDirectory',
            params={'clientInfo': clientInfo, 'publishedObjTagsAsString': publishedObjTagsAsString, 'clientLocale': clientLocale},
            response_cls=DescribePublicationRecordsResponse,
        )



from tcsoa.gen.ImportExport._2011_06.services import FileImportExportService as imp0
from tcsoa.gen.ImportExport._2007_06.services import FileImportExportService as imp1
from tcsoa.gen.ImportExport._2008_06.services import FileImportExportService as imp2
from tcsoa.gen.ImportExport._2017_11.services import FileImportExportService as imp3
from tcsoa.gen.ImportExport._2012_09.services import FileImportExportService as imp4
from tcsoa.base import TcService


class FileImportExportService(TcService):
    createReqMarkup = imp0.createReqMarkup
    exportToApplication = imp1.exportToApplication
    exportToApplication2 = imp2.exportToApplication
    exportToApplication3 = imp0.exportToApplication
    exportToApplication4 = imp3.exportToApplication
    getExportTemplates = imp2.getExportTemplates
    getExportTemplates2 = imp0.getExportTemplates
    importFromApplication = imp1.importFromApplication
    importFromApplication2 = imp2.importFromApplication
    importFromApplication3 = imp0.importFromApplication
    importFromApplication4 = imp4.importFromApplication
    updateReqMarkup = imp0.updateReqMarkup



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class LOVData(TcBaseObj):
    """
    This structure is used to store the LOV information (valueProperty and descriptionProperty) as well as the filter
    properties.This structure is used to provide the property names as input and in the result this same structure used
    to store the values of the input properties.
    
    :var valueProperty: The property name which is used as dynamic LOV value
    :var descriptionProperty: The property name which is used as dynamic LOV Value description.
    :var filterProperties: A list of property names which are used as a filter property.
    """
    valueProperty: str = ''
    descriptionProperty: str = ''
    filterProperties: List[str] = ()


@dataclass
class LOVSearchResults(TcBaseObj):
    """
    This structure contains the LOV results from the executeDynamicLOVQuery operation as well as the list of
    unprocessed UIDs.
    
    :var lovValues: The list of LOV Values, their description and values of all filter attributes.
    :var unprocessedUIDs: This is a list of unprocessed UIDs. The LOV ITK returns the number of results defined in the
    LovFilterData:: maxResult variable but out of these results LovFilterData:: numberToLoad number of results get
    processed and the rest of the result get stored in the list and sent back to the client. These unprocessed UIDs
    will be set as input in the next consecutive call of service operation to process next set of data.
    :var serviceData: The Service Data.
    """
    lovValues: List[LOVData] = ()
    unprocessedUIDs: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class LovFilterData(TcBaseObj):
    """
    This structure contains the information based on which the query results will be displayed in the test
    functionality. This structure contains the user entered filter string based on which query results will be filtered
    out as well as the property name based on which the result will be sorted.
    
    :var filterString: The filter string which is used to filter out the query output.
    :var maxResult: The maximum number of result to be processed.
    :var numberToLoad: The number of result to be loaded in a single processing.
    :var sortPropertyName: The name of the property the LOV search result will be sorted.
    :var sortOrder: The sorting order. A value of 1 indicates sort property values to be in ascending order. A value of
    2 indicates sort property values to be in descending order. 0 is to indicate no sort.
    """
    filterString: str = ''
    maxResult: int = 0
    numberToLoad: int = 0
    sortPropertyName: str = ''
    sortOrder: int = 0


@dataclass
class QueryData(TcBaseObj):
    """
    This structure contains the query string formed by query criteria, LOVData object which contains the LOV object
    name (valueProperty), LOV object description (descriptionProperty), list of filter properties provided by user and
    number of objects to be processed. This structure defines the input to the service operation
    executeDynamicLOVQuery. It contains LOV information as lovInputData, query definition as queryClass and
    queryString. This data structure contains a list of unprocessed UIDs. This list will be empty while a new query is
    going to be executed through the service operation. As an output service operation returns back the list of
    unprocessed UIDs to client which is eventually passed as an input parameter while displaying next set of LOV
    information.
    
    :var queryClass: This the business object class name on which the query will be generated. This represents the
    table name in database on which the query will be executed.
    :var queryString: This is a SQL style query string  formed by query criteria. This query string will contain query
    clauses as well as the table name on which the query will be executed.
    :var lovInputData: This variable stores the LOV information like the property name to represent LOV value and LOV
    description and a list of property name which is used as a filter property.
    :var filterData: It contains the user entered search string, number of result to be processed and sorting property
    name.
    :var unprocessedUIDs: This is a list of unprocessed UIDs. This list will be empty while the service operation is
    going to execute a query(in case of change in any other information in QueryData structure) and it will contain the
    list of unprocessed UIDs from the previous call of the service operation if the service operation is called only to
    process next set of unprocessed UIDs.
    """
    queryClass: str = ''
    queryString: str = ''
    lovInputData: LOVData = None
    filterData: LovFilterData = None
    unprocessedUIDs: List[str] = ()



from __future__ import annotations

from tcsoa.gen.ClassificationCommon._2020_01.Classification import ClassificationObjectResponse
from tcsoa.base import TcService


class ClassificationService(TcService):

    @classmethod
    def deleteClassificationObjects(cls, jsonRequest: str) -> ClassificationObjectResponse:
        """
        Deletes one or more classification objects permanently. A classification object is also called ICO. The
        classified workspace object associated with the ICO will not be deleted.
        
        Use cases:
        User needs to delete classification objects. It is typically called when after creating or searching the
        classification objects, user decides that the returned objects are not needed anymore.
        """
        return cls.execute_soa_method(
            method_name='deleteClassificationObjects',
            library='ClassificationCommon',
            service_date='2020_01',
            service_name='Classification',
            params={'jsonRequest': jsonRequest},
            response_cls=ClassificationObjectResponse,
        )

    @classmethod
    def getClassificationObjects(cls, jsonRequest: str) -> ClassificationObjectResponse:
        """
        Looks for specified classification objects. If they are found, then detailed information about those objects is
        provided.
        
        Use cases:
        This operation can be utilized when user needs to find an existing classification object(s) to either view or
        update its details. It can be followed by operations like saveClassificationObjects() or
        deleteClassificationObjects() to update or delete the classification objects.
        This operation, getClassificationObjects() is used to get detailed information about the specified
        classification objects.
        """
        return cls.execute_soa_method(
            method_name='getClassificationObjects',
            library='ClassificationCommon',
            service_date='2020_01',
            service_name='Classification',
            params={'jsonRequest': jsonRequest},
            response_cls=ClassificationObjectResponse,
        )

    @classmethod
    def saveClassificationObjects(cls, jsonRequest: str) -> ClassificationObjectResponse:
        """
        Creates one or more classification objects and (optionally) attaches them to a workspace object, thus
        classifying it. If the Classification objects are not associated with any workspace object, they would act as
        standalone Classification objects. A classification object is also called ICO.
        
        Use cases:
        User wants to classify a workspace object or create a standalone classification object (ICO) in Teamcenter.
        This operation expects a json string and supports various options such as associating the classification object
        with a workspace object, populating values of any attributes associated with classification objects and so on. 
        Before creating classification objects, a classification class hierarchy should already be created by the
        classification admin user in Teamcenter. This hierarchy should include a storage class (a class that allows
        instances to be created and associated to it) for which the classification objects need to be created.
        """
        return cls.execute_soa_method(
            method_name='saveClassificationObjects',
            library='ClassificationCommon',
            service_date='2020_01',
            service_name='Classification',
            params={'jsonRequest': jsonRequest},
            response_cls=ClassificationObjectResponse,
        )



from __future__ import annotations

from tcsoa.gen.ProjectManagement._2014_10.ScheduleManagement import FailedObjectContainer, EVMDataRequest, AnalyticsMultipleResourceAssignmentStacks, InsertScheduleContainer, LoadResourceGraphContainer, EVMResultsResponse, DetachScheduleContainer, CostDetailResponse
from tcsoa.gen.ProjectManagement._2012_09.ScheduleManagement import AssignmentCreateContainer
from tcsoa.gen.ProjectManagement._2012_02.ScheduleManagement import CreatedObjectsContainer, TaskCreateContainer
from typing import List
from tcsoa.gen.BusinessObjects import ScheduleTask, Schedule, ResourceAssignment
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def getResourceGraphData(cls, loadResourceGraphContainer: LoadResourceGraphContainer) -> AnalyticsMultipleResourceAssignmentStacks:
        """
        The operation calculates the resource load information of one or more users assigned to ScheduleTask. The
        operation takes in a list of User objects as the input for whom the resource graph data is generated. You can
        specify the start date and end date to get the resource loading over the period of time specified by the start
        and end dates. The operation takes into account all the ScheduleTask objects in the system where the User is
        assigned to compute the resource load.
        
        Use cases:
        Create resource assignments histogram in Schedule Manager and Teamcenter Organization.
        
        Exceptions:
        >Error Severity: 
        Information - User has no task assignments.
        Error - User Uid is not valid
        """
        return cls.execute_soa_method(
            method_name='getResourceGraphData',
            library='ProjectManagement',
            service_date='2014_10',
            service_name='ScheduleManagement',
            params={'loadResourceGraphContainer': loadResourceGraphContainer},
            response_cls=AnalyticsMultipleResourceAssignmentStacks,
        )

    @classmethod
    def insertSchedule(cls, insertScheduleContainer: List[InsertScheduleContainer]) -> ServiceData:
        """
        This operation inserts one or more Schedule specified in the input in a master Schedule. 
        You can specify in the input an optional parameter which if true, the operation will adjust the master Schedule
        start date and end date to be the same as the sub Schedule start date and end date. By default, the operation
        does not adjust the dates of the master Schedule to be the same as the sub Schedule dates.
        """
        return cls.execute_soa_method(
            method_name='insertSchedule',
            library='ProjectManagement',
            service_date='2014_10',
            service_name='ScheduleManagement',
            params={'insertScheduleContainer': insertScheduleContainer},
            response_cls=ServiceData,
        )

    @classmethod
    def replaceAssignment(cls, schedule: Schedule, newAssignments: List[AssignmentCreateContainer], assignmentDeletes: List[ResourceAssignment]) -> CreatedObjectsContainer:
        """
        This operation replaces a specified list of existing ResourceAssignment objects on a ScheduleTask with a
        specified list of new ResourceAssignment. The orginal ResourceAssignment specified in the input are deleted and
        new ResourceAssignment specified in the input are created on the ScheduleTask
        
        
        Use cases:
         In Schedule Manager, right click tasks and select Assignments->Replace Assignments.
        A dialogue will appear with current resources on the left hand side and the list of resources on the right hand
        side to replace the existing ones with. Select the resources on both the sides and click OK.
        
        Exceptions:
        >Service Exception when the resource assignments cannot be added on the task due to workflow process being
        already initiated on the ScheduleTask or there is no access privilege on the ScheduleTask
        
        230065     Resource assignments cannot be added, updated or deleted, and privileged user cannot be changed
        since the workflow process has been triggered at another site.
        230045     The user does not have the correct access privileges to modify one or more objects.
        230061     The schedule has been deleted and cannot be accessed.
        """
        return cls.execute_soa_method(
            method_name='replaceAssignment',
            library='ProjectManagement',
            service_date='2014_10',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'newAssignments': newAssignments, 'assignmentDeletes': assignmentDeletes},
            response_cls=CreatedObjectsContainer,
        )

    @classmethod
    def verifySchedule(cls, schedule: Schedule) -> FailedObjectContainer:
        """
        This operation will verify the input Schedule for integrity of the scheduling data. The operation validates all
        the ScheduleTask, ResourceAssignment objects, TaskDependency, constraints in the Schedule for their effort
        driven scheduling values. It is recommended to use this operation to verify the integrity and data correctness
        of the Schedule before publishing the Schedule for execution.
        
        Use cases:
         
        
        Exceptions:
        >Service Exception when there are failures in retrieving the properties of the ScheduleTask or failures in
        retrieving the ScheduleTask of the Schedule.
        
        230040  Getting task properties failed. See system log for details.
        230041  Getting tasks in schedule failed. See system log for details.
        """
        return cls.execute_soa_method(
            method_name='verifySchedule',
            library='ProjectManagement',
            service_date='2014_10',
            service_name='ScheduleManagement',
            params={'schedule': schedule},
            response_cls=FailedObjectContainer,
        )

    @classmethod
    def createPhaseGateTask(cls, taskInputContainer: TaskCreateContainer) -> CreatedObjectsContainer:
        """
        This operation creates a phase gate structure inside a Schedule. A phase gate structure comprises of a phase
        task, two milestones, a gate task and a finish to start TaskDependency between the phase and the gate tasks.
        The operation takes the start date,  finish date, work estimate of the phase task as input and computes the
        start and finish dates of the milestones and the gate task. The name sent in the input will be used for the
        names of the phase and gate tasks with the string "Phase" and "Gate" appended.
        """
        return cls.execute_soa_method(
            method_name='createPhaseGateTask',
            library='ProjectManagement',
            service_date='2014_10',
            service_name='ScheduleManagement',
            params={'taskInputContainer': taskInputContainer},
            response_cls=CreatedObjectsContainer,
        )

    @classmethod
    def detachSchedule(cls, detachScheduleContainer: List[DetachScheduleContainer]) -> ServiceData:
        """
        This operation removes specified sub Schedule from a given master Schedule. It takes in as input the the sub
        Schedule that needs to be removed and the master Schedule from where the sub Schedule needs to be removed.
        """
        return cls.execute_soa_method(
            method_name='detachSchedule',
            library='ProjectManagement',
            service_date='2014_10',
            service_name='ScheduleManagement',
            params={'detachScheduleContainer': detachScheduleContainer},
            response_cls=ServiceData,
        )

    @classmethod
    def getCostRollupData(cls, costDetailRequest: List[ScheduleTask]) -> CostDetailResponse:
        """
        The operation calculates the total cost associated with either a Schedule or ScheduleTask. The total cost of a
        Schedule or ScheduleTask is calculated as below
        1) The total of the fixed costs specified on the Schedule or ScheduleTask
        2) The total of the costs of each ResourceAssignment based on the work hours and resource rate reported against
        the ScheduleTask.
        If there are any child ScheduleTask below the input ScheduleTask, the costs of the child ScheduleTask are
        accumulated into the costs of the parent ScheduleTask.
        """
        return cls.execute_soa_method(
            method_name='getCostRollupData',
            library='ProjectManagement',
            service_date='2014_10',
            service_name='ScheduleManagement',
            params={'costDetailRequest': costDetailRequest},
            response_cls=CostDetailResponse,
        )

    @classmethod
    def getEVMResults(cls, inputEVMData: EVMDataRequest) -> EVMResultsResponse:
        """
        The operation calculates the earned valued results for Schedule or ScheduleTask. The operation takes in input
        the Schedule or ScheduleTask  on which the earned value calculated data is sought and the criteria for the
        calculations. The earned value is calculated for the following labels -
        Planned Value (PV), Earned Value (EV), Actual Cost (AC) or Actual Cost of Work Performed, (ACWP) (Actual
        Effort), Cost Variance (CV), Budget at Completion (BAC), Forecast at Completion (FAC), Estimate at Completion
        (EAC), Schedule Variance (SV), Schedule Performance Index (SPI),Cost Performance Index (CPI)
        """
        return cls.execute_soa_method(
            method_name='getEVMResults',
            library='ProjectManagement',
            service_date='2014_10',
            service_name='ScheduleManagement',
            params={'inputEVMData': inputEVMData},
            response_cls=EVMResultsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Cfg0AbsValue, Cfg0AbsFamily, RevisionRule, Cfg0ConfiguratorPerspective, Cfg0AbsConfiguratorWSO
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ValidateProductConfigOutput(TcBaseObj):
    """
    It is structure containing output information per input expressionsToValidate.
    
    :var updatedExpression: An expanded expression satisfying the given expression to validate and existing set of
    constraint in the configurator perspective.
    :var violations: The list of constraints that were violated for a given expression to validate.
    :var criteriaStatus: The status of the configuration. Valid  values  are: validAndComplete, validAndInComplete or
    inValid.  
    validAndComplete: all mandatory families in configuration have value selections.  
    validAndInComplete: not all mandatory families in configuration have value selections. 
    inValid: there are some violations .
    :var requiredFamilies: A list of mandatory families, which do not have any value selection based on input
    expression and default value in the context.
    """
    updatedExpression: BusinessObjectConfigExpression = None
    violations: List[Violation] = ()
    criteriaStatus: str = ''
    requiredFamilies: List[Cfg0AbsFamily] = ()


@dataclass
class ValidateProductConfigurationResponse(TcBaseObj):
    """
    The response structure for the operation ValidateProductConfiguration 
    
    :var serviceData: The service data for errors and returned objects.
    :var outputs: The list of expressions that was validated and updated with selections due to application of
    constraints.
    """
    serviceData: ServiceData = None
    outputs: List[ValidateProductConfigOutput] = ()


@dataclass
class AvailableProductVariabilityOutput(TcBaseObj):
    """
    The response structure returned by getAvailableProductVariability.
    
    :var availabilityExpressions: A list of AvailableVariability expressions defining available variability. The number
    of elements in this list should be equal to the number of  families which at least have a one available feature or
    default feature for the given input expression(criteriaExpression ). If an empty familiesToTest list was specified
    this vector would be empty.
    :var violations: A list of constraints that were violated for a given user selected values/input expression.
    :var suggestedSatisfyingExpr: A sample expression satisfying the given input expression and existing set of
    constraint in the configurator context. This is an output expression containing system suggested features from the
    available features.
    :var criteriaStatus: The status of the configuration. Valid values are: validAndComplete, validAndInComplete or
    inValid.  
    validAndComplete: all mandatory families in configuration have value selections.  
    validAndInComplete: not all mandatory families in configuration have value selections. 
    inValid: there are some violations.
    :var requiredFamilies: A list of mandatory families, which do not have any feature selection based on input
    expression and default feature in the context.
    :var serviceData: The service data for errors and returned objects.
    """
    availabilityExpressions: List[AvailableVariability] = ()
    violations: List[Violation] = ()
    suggestedSatisfyingExpr: ApplicationConfigExpression = None
    criteriaStatus: str = ''
    requiredFamilies: List[Cfg0AbsFamily] = ()
    serviceData: ServiceData = None


@dataclass
class Violation(TcBaseObj):
    """
    Represents a violated rule along with the message and severity associated with rule violation.
    
    :var message: The message to display if this rule is violated.
    :var severity: The severity associated with the message. Valid values for this parameter are defined by the
    ListOfValues "Cfg0ConstraintSeverity".
    :var violatedRule: The rule that was found to be violated.
    :var configuratorWSO: Validation constraint for product variant configurations. It might contain NULL in case user
    is not privileged to read validation constraint rule.
    """
    message: str = ''
    severity: str = ''
    violatedRule: ApplicationConfigExpression = None
    configuratorWSO: Cfg0AbsConfiguratorWSO = None


@dataclass
class AvailableVariability(TcBaseObj):
    """
    Defines available features for a given family along with the default features for operation
    getAvailableProductVariability. The AvailableVariability structure is interpreted as "NULL" if it has the following
    values set for its parameters:
    
    defaultValues: empty list
    
    availability: empty list
    
    :var defaultValues: Specifies the list of configuration expression terms referencing default features from the
    requested family. This list can have more than one element only if the family is a multi-select otherwise list
    would contain either zero or one element. If no default rules exist or they select a default feature such that it
    is ruled out due to other constraints, this will be an empty list to indicate  non-existing default values.
    :var availability: The list of configuration expressions referencing values only from the requested families. This
    is a list of discrete values or ranges.
    """
    defaultValues: List[ConfigExpressionTerm] = ()
    availability: List[ConfigExpressionTerm] = ()


@dataclass
class BusinessObjectConfigExpression(TcBaseObj):
    """
    Relates a business object (e.g. Cpd0DesignElement, VariantRule, or Cfg0ExcludeRule) to a list of configuration
    expressions.
    
    :var targetObject: The business object (e.g. Cpd0DesignElement, VariantRule, or Cfg0ExcludeRule), to which the
    configuration expressions specified in parameter 'expressions' are linked.
    :var expressions: The list of 'ApplicationConfigExpression' structures for the business object.
    :var clientId: The unique identifier to distinguish the input and corrosponding output.
    """
    targetObject: BusinessObject = None
    expressions: List[ApplicationConfigExpression] = ()
    clientId: str = ''


@dataclass
class ConfigExprWithDisplayString(TcBaseObj):
    """
    It contains the config expression populated with formula, SOA grid structure and display format.
    
    :var configExprDisplayString: Display string for configuration expression.
    """
    configExprDisplayString: str = ''


@dataclass
class ApplConfigExprWithDisplayString(TcBaseObj):
    """
    It contains the application configuration expression populated with formula,  grid structure and display format.
    
    :var applConfigExpr: The application configuration expression for which formula,  grid structure and display format
    is requested.
    :var applConfigExprDisplayString: Display string for application configuration expression.
    :var configExprSetsInfo: List of configuration expression set which will contain list of configuration expressions
    populated with formula,  grid structure and display format.
    """
    applConfigExpr: ApplicationConfigExpression = None
    applConfigExprDisplayString: str = ''
    configExprSetsInfo: List[ConfigExpressionSetInfo] = ()


@dataclass
class ConfigExpression(TcBaseObj):
    """
    Defines a Boolean expression to be used for configuration purposes.
    
    :var clientID: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var expressionType: The expression category and the intent of this expression.
    
    The valid values are as follows:
    - 9   - Variant Condition
    - 18 - Variant Rule
    - 25 - Applicability
    - 44 - Subject
    
    
    :var formula: The persistent formula string in Teamcenter syntax (see documentation for property
    ''cfg0SubjectCondition'' or BOMLine property ''bl_formula'').
    :var subExpressions: The list of sub-expressions in a Boolean variant expression. The sub-expressions are joined
    together by OR operator.
    """
    clientID: str = ''
    expressionType: int = 0
    formula: str = ''
    subExpressions: List[ConfigSubExpression] = ()


@dataclass
class ConfigExpressionGroup(TcBaseObj):
    """
    This structure groups 'ConfigExpressionTerm' structures. A mixture of positive and negated expression terms in the
    same expression group is not allowed. Positive terms within an expression group are combined with operator 'OR',
    e.g. Color=Red | Color=Green. Negated terms within an expression group are combined with operator 'AND', e.g.
    Color!=Red & Color!=Green.
    
    :var context: The family object instance to be used in a free form family expression term, or a feature range
    expression.
    :var groupName: The name for the group containing the expression terms. If the group name is provided as FALSE then
    group expression would be considered as FALSE. If the group name is provided as TRUE then group expression would be
    considered as TRUE.
    :var terms: the list of 'ConfigExpressionTerm' structures. The terms are combined together by OR operator.
    """
    context: Cfg0AbsFamily = None
    groupName: str = ''
    terms: List[ConfigExpressionTerm] = ()


@dataclass
class ConfigExpressionSet(TcBaseObj):
    """
    The collection of 'ConfigExpressions' refered as set for the business object. A set collects different type of
    varint expressions such as applicability and subject. These will be joined together by AND operation and evaluated
    as part of single statement.
    
    :var configExpressions: The list of 'ConfigExpression' structures which are collected as a set for the business
    object. Each structure defines a variant expression which can be used for the configuration purposes.
    """
    configExpressions: List[ConfigExpression] = ()


@dataclass
class ConfigExpressionSetInfo(TcBaseObj):
    """
    It contains the list of config expressions populated with formula, SOA grid structure and display format.
    
    :var configExprsWithDisplayString: List of configuration expressions populated with formula,  grid structure and
    display format.
    """
    configExprsWithDisplayString: List[ConfigExprWithDisplayString] = ()


@dataclass
class ConfigExpressionTerm(TcBaseObj):
    """
    Defines an elemental expression literal for a single feature, or feature range expression for a given family.
    
    :var family: The family object instance to be used in a free form family expression term, or a feature range
    expression.
    :var familyID: The ID of the family in the term
    :var familyNamespace: The family namespace by which the family object is uniquely identified.
    :var operatorCode: The operator code to use for this expression terms.
    
    The Valid values are as follows:
    - 5 - Equals
    - 6 - Not Equals
    - 10 - And
    - 11 - OR
    
    
    :var rangeExpressions: A list of elemental expression literals describing a feature range for a given  family.
    :var value: The feature object to be used in an elemental expression literal.
    :var valueText: The ID of the feature object or the feature to use for a free form family.
    :var selectionClass: Specifies the selection type of the feature object. This parameter specifies whether the
    feature object is user selected, default selected or system selected.
    
    Valid Values are as follow:
    - EmptyInitialized A constraint has initialized a family with the empty value.
    - Default          A constraint has assigned a default value.
    - PackageDefault   A package constraint has assigned a default value.
    - Unknown          An administrator authored constraint with unknown severity has assigned a value. If a feature
    selection is not known precisely, it will be represented as unknown.
    - Information      A constraint with Info severity has assigned a value. Refer Cfg0ConstraintSeverity list of
    values for details.
    - Warning          A constraint with Warning severity has assigned a value. Refer Cfg0ConstraintSeverity list of
    values for details.
    - Error            A constraint with Error severity has assigned a value. Refer Cfg0ConstraintSeverity list of
    values for details.
    - User             User has assigned a value.
    
    """
    family: Cfg0AbsFamily = None
    familyID: str = ''
    familyNamespace: str = ''
    operatorCode: int = 0
    rangeExpressions: List[RangeExpression] = ()
    value: Cfg0AbsValue = None
    valueText: str = ''
    selectionClass: str = ''


@dataclass
class ApplicationConfigExpression(TcBaseObj):
    """
    The configuration expression for the business object.
    
    :var formula: The persistent formula string in Teamcenter syntax.
    :var exprID: The application expression identifier used to distinguish the current expression from other variant
    expressions authored for the same business object.
    :var expressionType: Specifies the expression category and the intent of this expression. The valid values are as
    follows:
    -  9 -  Variant Condition
    - 18 - Variant Rule
    - 28 - Inclusive Constraint
    - 29 - Exclusive Constraint
    - 37 - Default Rule
    - 41 - Availability Rule
    
    
    :var configExprSets: The list of 'ConfigExpressionSet' which are relevant to the business object. The variant
    expression sets are combined by OR operator.
    """
    formula: str = ''
    exprID: str = ''
    expressionType: int = 0
    configExprSets: List[ConfigExpressionSet] = ()


@dataclass
class ConfigSubExpression(TcBaseObj):
    """
    This structure lists the 'ConfigExpressionGroup' structures which are joined together by "AND" operator.
    
    :var expressionGroups: The list of clauses in a Boolean expression. The groups are combined together by AND
    operator.
    """
    expressionGroups: List[ConfigExpressionGroup] = ()


@dataclass
class ConvertVariantExpressionInput(TcBaseObj):
    """
    An input structure defining the list of application configuration expressions for which formula,  grid structure
    and the display format is requested.
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var exprsToConvert: List of application config expressions for which fomula, SOA grid struture and the display
    format is requested.
    :var context: The Cfg0ConfiguratorPerspective instance to specify the context and revision rule. This is a
    mandatory parameter.
    :var expressionFormat: This flag will indicate what will be populated in output 'ApplicationConfigExpression'
    structure. Following are valid values:
    0 - formulae, grid expression and display formulae
    1 - formulae
    2 - grid expression 
    3 - formulae and grid expression
    4 - display formulae 
    5 - formulae and display formulae
    6 - grid expression and display formulae
    """
    clientId: str = ''
    exprsToConvert: List[ApplicationConfigExpression] = ()
    context: Cfg0ConfiguratorPerspective = None
    expressionFormat: int = 0


@dataclass
class ConvertVariantExpressionOutput(TcBaseObj):
    """
    It contains the list of application config expressions populated with formula, SOA grid structure and display
    format.
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with the input
    structure.
    :var applConfigExprsWithDisplayString: List of application configuration expression populated with formula,  grid
    structure and display format.
    """
    clientId: str = ''
    applConfigExprsWithDisplayString: List[ApplConfigExprWithDisplayString] = ()


@dataclass
class ConvertVariantExpressionsResponse(TcBaseObj):
    """
    The response structure for the operation with the converted variant expressions and in required formats.
    
    :var outputs: The list of converted variant expressions.
    :var serviceData: Service Data
    """
    outputs: List[ConvertVariantExpressionOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetProductDefaultsConfigOutput(TcBaseObj):
    """
    The list of expressions that was validated and status of configuration along with required families.
    
    :var expressionWithDefaults: The list of output variant expressions that result from applying default rules to the
    input variant expressions.
    :var criteriaStatus: The status of the configuration. Valid values are: validAndComplete, validAndInComplete or
    inValid.  
    validAndComplete: all mandatory families in configuration have value selections.  
    validAndInComplete: not all mandatory families in configuration have value selections. 
    inValid: there are some violations.
    :var requiredFamilies: A list of mandatory families, which do not have any value selection based on input
    expression and default feature in the context.
    """
    expressionWithDefaults: BusinessObjectConfigExpression = None
    criteriaStatus: str = ''
    requiredFamilies: List[Cfg0AbsFamily] = ()


@dataclass
class GetProductDefaultsResponse(TcBaseObj):
    """
    The response structure for operation GetProductDefaults.
    
    :var outputs: The list of the output expressions that result from applying default constraints to the input
    expressions.
    :var serviceData: The service data to return any additional information.
    """
    outputs: List[GetProductDefaultsConfigOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetRevisionRulesResponse(TcBaseObj):
    """
    The list of applicable RevisionRule objects with the service data.
    
    
    
     
    
    :var applicableRevisionRules: The list of RevisionRule instances.
    :var serviceData: The teamcenter service data
    """
    applicableRevisionRules: List[RevisionRule] = ()
    serviceData: ServiceData = None


@dataclass
class GetVariantExpressionsResponse(TcBaseObj):
    """
    The response structure for operation 'getVariantExpressions'.
    
    :var configObjectExpressions: A list of structures each relating a business object to a set of configuration
    expressions.
    :var serviceData: The Teamcenter service data
    """
    configObjectExpressions: List[BusinessObjectConfigExpression] = ()
    serviceData: ServiceData = None


@dataclass
class ProductDefaultsInput(TcBaseObj):
    """
    Input structure for the getProductDefaults operation.
    
    :var applyDefaults: The default actions to apply. The parameter is a bitwise OR the following values: 
    - 0 - Indicates that defaults should not be applied 
    - 1 - Will enable application of defaults 
    - 2 - Ignore all derived defaults and only process fixed defaults 
    - 4 - Skip derived defaults whose applicability or product model condition is not fully and explicitly satisfied.
    For example, a default rule 'set W=10 if H=10' is skipped for the input expression 'L=10' because 'L=10' also
    includes configurations that don't satisfy the applicability condition, such as 'L=10 & H=20'.
    - 8 - Force partially satisfied derived defaults.
    
    
    :var expressionsToUpdate: The list of expressions for which defaults are requested to be applied.
    :var families: This is an optional parameter. If specified, the server returns defaults only for these families. If
    null, the server returns defaults for all families
    :var context: The Cfg0ConfiguratorPerspective instance to provide the  configurator context(s) and RevisionRule. It
    may also include configuration information e.g. a variant rule. Expression with Variant Rule is used as additional
    criteria for validation.
    """
    applyDefaults: int = 0
    expressionsToUpdate: List[BusinessObjectConfigExpression] = ()
    families: List[Cfg0AbsFamily] = ()
    context: Cfg0ConfiguratorPerspective = None


@dataclass
class RangeExpression(TcBaseObj):
    """
    An elemental expression literal that is used in a value range expression for a given family. 
    Note: Operator PS_variant_operator_is_equal is not supported (use 'ConfigExpressionTerm' structures for equality
    literals).
    
    
    :var operatorCode: The operator code by which the value should be used in variant expression.
    :var valueText: The string value of the feature.
    :var feature: This is an optional parameter. If specified, the server and client should use it. If null, the server
    and client should work with valueText. Specifies feature object to be used in an range expression.
    """
    operatorCode: int = 0
    valueText: str = ''
    feature: Cfg0AbsValue = None


@dataclass
class AvailableProductVariabilityInput(TcBaseObj):
    """
    It is the object containing input information required to compute the available product variability.
    
    :var criteriaExpression: The input criteria expression for which the available product variability is to be
    calculated.
    :var familiesToTest: The list of configuration families of which available features are requested.
    :var context: Configurator context details.
    :var applyConstraints: How to apply the constraints. The action is a bitwise OR the following values: 
    0: disable configurator constraint evaluation 
    1: enable configurator constraint evaluation 
    2: skip constraints if they only reference unset families 
    4: skip constraints that don&apos;t reference all configExpression families 
    8: report warnings in addition to errors 
    16: report informational messages 
    32: don&apos;t stop if a violation is encountered (use with care) 
    64: treat constraints with warning severity as if they had error severity.
    
    1024: skip criteria update based on validation rules (For example configurator exclusion rule). If this flag is not
    set then the operation will update the variant criteria after applying validation rules.
    
    2048: enable Availability Constraint evaluation. 
    """
    criteriaExpression: BusinessObjectConfigExpression = None
    familiesToTest: List[Cfg0AbsFamily] = ()
    context: Cfg0ConfiguratorPerspective = None
    applyConstraints: int = 0


@dataclass
class SetVariantExpressionInput(TcBaseObj):
    """
    'SetVariantExpressionInput' represents a single transaction and partial error boundary for setting expressions on a
    list of target business objects.
    
    :var businessObjectExpressions: The list of business objects and corrosponding expressions.
    :var revisionRule: The revision rule to use to retrieve the correct revisions of the option data referenced by the
    expressions.
    :var saveExpressions: The flag which indicates if the configuration expressions should be saved on the target
    objects or set as transient configuration expressions. If an transient configuration expression is create, then
    user need to save the target object explicitly.
    """
    businessObjectExpressions: List[BusinessObjectConfigExpression] = ()
    revisionRule: RevisionRule = None
    saveExpressions: bool = False


@dataclass
class ValidateProductConfigInput(TcBaseObj):
    """
    It is structure containing input information required to validate product configuration.
    
    :var applyConstraints: The constraint rules application. The parameter is a bitwise OR of the following values: 
    - 0: disable configurator constraint evaluation 
    - 1: enable configurator constraint evaluation 
    - 2: skip constraints if they only reference unset families 
    - 4: skip constraints that don't reference all Expression families 
    - 8: report warnings in addition to errors 
    - 16: report informational messages 
    - 32: Continue even if a violation is encountered 
    - 64: treat constraints with warning severity as if they had error severity.
    - 1024: skip criteria update based on validation rules (For example configurator exclusion rule). If this flag is
    not set then the operation will update the variant criteria after applying validation rules.
    - 2048: enable Availability Constraint evaluation.
    
    
    :var expressionsToValidate: The input criteria expression for which the validation is requested.
    :var context: The Cfg0ConfiguratorPerspective instance which provides the RevisionRule and the configurator context.
    """
    applyConstraints: int = 0
    expressionsToValidate: List[BusinessObjectConfigExpression] = ()
    context: Cfg0ConfiguratorPerspective = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import DatasetType, ImanType
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetDatasetTypesWithDefaultRelOutput(TcBaseObj):
    """
    This GetDatasetTypesWithDefaultRelationOutput struct contains the list dataset type, their default paste relation
    for the given parent and reference information and Service Data.
    
    :var fileExtension: The fileExtension for each Dataset type specified in fileExtensions input.
    :var datasetTypesWithDefaultRelInfo: The matching list of named reference information and default paste relation
    for the Dataset type.
    """
    fileExtension: str = ''
    datasetTypesWithDefaultRelInfo: List[DatasetTypeInfoWithDefaultRelation] = ()


@dataclass
class ReferenceInfo(TcBaseObj):
    """
    The ReferenceInfo struct contains information for a given Dataset type
    
    :var referenceName: The reference name for the input dataset type.
    :var isObject: If true, signifies the reference is an object. False signifies the reference is a file.
    :var fileFormat: The format for reference object. Valid values are either TEXT, BINARY or OBJECT.
    :var fileExtension: The default extension for a file, such as *.gif or *.doc.
    """
    referenceName: str = ''
    isObject: bool = False
    fileFormat: str = ''
    fileExtension: str = ''


@dataclass
class DatasetTypeInfoWithDefaultRelation(TcBaseObj):
    """
    The DatasetTypeInfoWithDefaultRelation struct contains the dataset type object reference corresponding to the input
    dataset type name, default paste relation for the input parent and the reference information for each valid named
    reference of the dataset type.
    
    :var datasetType: The DatasetType object for the input dataset type
    :var refInfos: The list of valid named references for the input DatasetType.
    :var defaultRelationInfo: The default paste relation for the input DatasetType for the given input parent.
    """
    datasetType: DatasetType = None
    refInfos: List[ReferenceInfo] = ()
    defaultRelationInfo: DefaultRelationInfo = None


@dataclass
class DatasetTypesWithDefaultRelation(TcBaseObj):
    """
    This DatasetTypesWithDefaultRelation struct contains pair of file extension and the list of
    DatasetTypeInfoWithDefaultRelation structures.
    
    :var output: List of named reference information and default paste relation for the given input parent for each
    Dataset type specified in fileExtensions input.
    :var serviceData: The DatasetType objects that corresponds to fileExtensions input are added to the Plain object
    list.
    """
    output: List[GetDatasetTypesWithDefaultRelOutput] = ()
    serviceData: ServiceData = None


@dataclass
class DefaultRelationInfo(TcBaseObj):
    """
    The DefaultRelationInfo struct contains information for a given dataset type.
    
    :var defaultRelation: The ImanType of the default paste relation for the Dataset for the given input parent.
    :var name: Internal name of the default paste relation for the Dataset for the given input parent.
    :var displayName: Display name of the default paste relation for the Dataset for the given input parent.
    """
    defaultRelation: ImanType = None
    name: str = ''
    displayName: str = ''



from __future__ import annotations

from tcsoa.gen.Internal.BusinessModeler._2010_04.DataModelManagement import DataModelDeploymentInput, TemplateFilesResponse, TemplateFileInput, ChangedTemplateFileInput
from tcsoa.gen.Internal.BusinessModeler._2007_01.DataModelManagement import ImportDataModelResponse
from typing import List
from tcsoa.base import TcService


class DataModelManagementService(TcService):

    @classmethod
    def getTemplateFiles(cls, inputs: List[TemplateFileInput]) -> TemplateFilesResponse:
        """
        Template files store Teamcenter data model definition information. This operation gets the specified template
        ('inputs') related files from the respective server directories.  In a Teamcenter installed environment, all
        template files are located at '$(TC_DATA)\model' directory. This is an unpublished operation.
        
        Use cases:
        A user can get all templates that a custom template is dependent on from the server $(TC_DATA)\model directory
        using this operation
        """
        return cls.execute_soa_method(
            method_name='getTemplateFiles',
            library='Internal-BusinessModeler',
            service_date='2010_04',
            service_name='DataModelManagement',
            params={'inputs': inputs},
            response_cls=TemplateFilesResponse,
        )

    @classmethod
    def deployDataModel(cls, inputs: List[DataModelDeploymentInput], deployOption: str, updaterUpdateOption: str, updaterModeOption: str) -> ImportDataModelResponse:
        """
        This operation deploys the data model into the database using an XML file based on the input deploy and updater
        options.  Business model updater is used to update the database. Localization (language) files are also
        maintained through this operation.  This operation is deprecated. Please use 'deployDataModel2' Operation.
        """
        return cls.execute_soa_method(
            method_name='deployDataModel',
            library='Internal-BusinessModeler',
            service_date='2010_04',
            service_name='DataModelManagement',
            params={'inputs': inputs, 'deployOption': deployOption, 'updaterUpdateOption': updaterUpdateOption, 'updaterModeOption': updaterModeOption},
            response_cls=ImportDataModelResponse,
        )

    @classmethod
    def getChangedTemplateFiles(cls, inputs: List[ChangedTemplateFileInput], retrieveFiles: bool) -> TemplateFilesResponse:
        """
        Detects if any of the given template related files ('inputs') have changed. Gets the changes template files if
        the 'retriveFiles' option set to 'true'.
        """
        return cls.execute_soa_method(
            method_name='getChangedTemplateFiles',
            library='Internal-BusinessModeler',
            service_date='2010_04',
            service_name='DataModelManagement',
            params={'inputs': inputs, 'retrieveFiles': retrieveFiles},
            response_cls=TemplateFilesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Group, Role, GroupMember
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetGroupsForRoleInput(TcBaseObj):
    """
    This structure hold a Role business object to find the matching groups and a boolean flag. Each matching groups
    should have this role in its role list.
    
    :var role: The given Role to find matching groups.
    :var includeOrgTree: Flag to indicate if an optional matched group tree structure would be returned. Set it to true
    if  the group tree structure should be sent back, false otherwise.
    """
    role: Role = None
    includeOrgTree: bool = False


@dataclass
class GetOrganizationGroupOutput(TcBaseObj):
    """
    Organization Group Ouput.
    
    :var groups: List of matched groups.
    :var orgTree: Tree structure of the matched groups and their parent groups. It could be null if optional input flag
    is set to not return this tree.
    """
    groups: List[Group] = ()
    orgTree: OrganizationTree = None


@dataclass
class GetOrganizationGroupResponse(TcBaseObj):
    """
    A list of GetOrganizationGroupOutput objects for each of the given Role object.
    
    :var outputs: List of GeyOrganizationGroupOutput objects, one for each GetGroupsForRoleInput object.
    :var serviceData: Object with all matched groups in plain object list and possible errors in finding matched groups.
    """
    outputs: List[GetOrganizationGroupOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GroupStructure(TcBaseObj):
    """
    This structure holds the group hierarchical information like the parent of a group and its child groups.
    
    :var group: A Group object which has the given role directly or through its descendent groups.
    :var parent: The parent group of the Group object.
    :var subgroups: Child groups of the Group object which contain the role directly or through their descendent groups.
    :var groupRoles: A List of roles for Group.
    """
    group: Group = None
    parent: Group = None
    subgroups: List[Group] = ()
    groupRoles: List[RoleUsers] = ()


@dataclass
class OrganizationTree(TcBaseObj):
    """
    This structure holds the list of top level groups and corresponding subgroups.
    
    :var rootLevelGroups: A list of  top level Group objects which have  the given role by itself or by their
    descendent groups.
    :var groupStructureMap: A lookup map to find corresponding GroupStructure for a given group. The key for the map 
    is BusinessObjectRef<Teamcenter::Group> and value is GroupStructure.
    """
    rootLevelGroups: List[GroupStructure] = ()
    groupStructureMap: GroupStructureMap = None


@dataclass
class RoleUsers(TcBaseObj):
    """
    This structure holds role and a list of  associated group members .
    
    :var role: The Role of a group member.
    :var members: List of group members for given role.
    """
    role: Role = None
    members: List[GroupMember] = ()


"""
A lookup map to find corresponding GroupStructure for a given group. The key for the map  is BusinessObjectRef<Teamcenter::Group> and value is GroupStructure.
"""
GroupStructureMap = Dict[Group, GroupStructure]



from tcsoa.gen.Diagramming._2014_06.services import DNDManagementService as imp0
from tcsoa.gen.Diagramming._2011_06.services import DiagramManagementService as imp1
from tcsoa.gen.Diagramming._2012_09.services import DiagramManagementService as imp2
from tcsoa.base import TcService


class DNDManagementService(TcService):
    createAndPaste = imp0.createAndPaste
    createConnectionPortsAndConnect = imp0.createConnectionPortsAndConnect


class DiagramManagementService(TcService):
    createDiagram = imp1.createDiagram
    createOrUpdateTemplate = imp1.createOrUpdateTemplate
    createOrUpdateTemplate2 = imp2.createOrUpdateTemplate
    getDiagramMembers = imp1.getDiagramMembers
    openDiagram = imp1.openDiagram
    saveDiagram = imp1.saveDiagram



from __future__ import annotations

from tcsoa.gen.UiConfig._2014_11.UiConfig import ColumnConfig, ScopeInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ResetUIConfigInput(TcBaseObj):
    """
    Contains input information required to reset UI configurations from the Teamcenter database.
    
    :var scope: The scope of the desired UI configuration information. This includes the name of the scope (i.e. a
    user, group or role) and scope query parameter information.
    :var columnConfigIds: List of column configuration IDs
    """
    scope: ScopeInput = None
    columnConfigIds: List[str] = ()


@dataclass
class ResetUIConfigResponse(TcBaseObj):
    """
    This structure returns information to the client about column configuration and command applicability. The
    ServiceData contains information about errors encountered during processing.
    
    :var serviceData: ServiceData structure containing errors and command, command collection and icon objects. If
    there is an error retrieving the configuration information, the error added to the ServiceData as a partial error.
    :var columnConfigurations: List of effective column configurations.
    """
    serviceData: ServiceData = None
    columnConfigurations: List[ColumnConfig] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetDynamicIPALinesResponse(TcBaseObj):
    """
    This structure is returned as response for getDynamicIPALines operation -
    It returns - 
    1. Map of input bop lines and their corresponding DIPA nodes.
    2. Partial errors as part of the serviceData. These errors will be those encountered during various aspects of
    traversal of process structure. 
    
    :var bopLineToIPALines: A map (BOMLine, list of BOMLline) where the key represents BOMLine of type Mfg0BvrProcess
    and value is list of BOMLine of type Mfg0BvrDynamicIPA. What constitutes a dynamic IPA node is defined by the
    preference 'MEDynamicIPAOccurrenceTypes', or lines below consumed lines as traversed by the closure rule specified
    by the preference 'MEDynamicIPALinesTraversalRule'.
    :var serviceData: The ServiceData.
    """
    bopLineToIPALines: BopLineToIPALinesMap = None
    serviceData: ServiceData = None


@dataclass
class CleanDynamicIPALinesInfo(TcBaseObj):
    """
    Structure containing information for cleaning the dynamic IPA lines.
    
    :var inputBOPLines: List of business objects representing BOP lines
    :var cleanSubHierarchy: Flag indicating whether to consider processes in sub-hierachy to clean the dynamic IPA
    liines
    """
    inputBOPLines: List[BusinessObject] = ()
    cleanSubHierarchy: bool = False


"""
A map (BOMLine, list of BOMLline) where the key represents BOMLine of type Mfg0BvrProcess and value is list of BOMLine of type Mfg0BvrDynamicIPA. What constitutes a dynamic IPA node is defined by the preference 'MEDynamicIPAOccurrenceTypes', or lines below consumed lines as traversed by the closure rule specified by the preference 'MEDynamicIPALinesTraversalRule'.
"""
BopLineToIPALinesMap = Dict[BusinessObject, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.Classification._2009_10.Classification import KeyLOVDefinition2, AttributeConfiguration
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Classification._2007_01.Classification import ClassificationProperty, TypedDocument
from enum import Enum
from tcsoa.gen.Internal.Classification._2017_05.Classification import FormatPropertiesNX
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExtendedPropertyInfo(TcBaseObj):
    """
    Extended metadata properties for an attribute.
    
    :var propName: An extended property name.
    :var propValue: Extended property values.
    """
    propName: str = ''
    propValue: List[str] = ()


@dataclass
class FindClassificationInfoRequest(TcBaseObj):
    """
    Input for FindClassificationInfo operation.
    
    :var workspaceObjects: List of input WorkspaceObjects for which classification information is required.
    :var relationTypes: GRM relation types used to get classification object information.
    Possible values are -
    &bull;IMAN_classification
    &bull;Fnd0SourceClassification
    """
    workspaceObjects: List[BusinessObject] = ()
    relationTypes: List[str] = ()


@dataclass
class FindClassificationInfoResponse(TcBaseObj):
    """
    Holds the classification objects returned by the findClassificationInfo operation.
    
    :var classificationObjectsInfoMap: Map (BusinessObject, std::vector< ClassificationObjectInformation >) of
    WorkspaceObject to corresponding Classification Object details.
    :var classInfoMap: A map (string, ClassInformation) of Classification class or view ID and its corresponding
    definition.
    :var keylovInfoMap: A map (string ID, KeyLOVDefinition2) containing the used Classification KeyLOVs (stxt) and its
    definitions.
    :var serviceData: Any failures will be returned in the service data list of partial errors.
    """
    classificationObjectsInfoMap: ClassificationObjectsInfoMap = None
    classInfoMap: ClassInfoMap = None
    keylovInfoMap: KeylovInfoMap = None
    serviceData: ServiceData = None


@dataclass
class ClassAttributeInfo(TcBaseObj):
    """
    Structure representing class attribute details.
    
    :var id: Unique ID for this attribute.
    :var name: Display name of the attribute.
    :var dependencyAttribute: The dependency attribute property of this attribute.
    :var dependencyConfiguration: The dependency configuration property of this attribute.
    :var extendedProperties: A list of references to the ExtendedProperty structure holding the extended metadata
    properties of this attribute.
    :var configuredKeyLOV: KeyLOV (stxt) information based on dependency configuration of an attribute.
    :var shortName: Short name defined for this attribute.
    :var description: Description added for this attribute.
    :var metricFormat: The metric format of this attribute.
    :var nonMetricFormat: The nonmetric format of this attribute.
    :var annotation: Annotation information added to this attribute.
    :var attributeConfiguration: Additional configuration for this attribute.
    :var arraySize: Array size or the number of values for this attribute. 
    &bull;    If single valued (non-array), then arraySize = 1.
    &bull;    If multi valued (array), then arraySize >= 1 corresponding to the size of the array defined in the
    attribute definition.
    :var options: Attribute property flags represented as a single integer. To access individual property flags, a
    bitwise operation will be required by the client. Valid values are:
    &bull;     vla = 1 
    &bull;     external_vla = 2
    &bull;     mandatory = 4 
    &bull;     protected = 8 
    &bull;     unique = 16 
    &bull;     propagated = 32 
    &bull;     localValue = 64 
    &bull;     reference = 128 
    &bull;     auto_computed = 32768 
    &bull;     hidden = 32768 
    &bull;     localizable = 4194304
    """
    id: int = 0
    name: str = ''
    dependencyAttribute: str = ''
    dependencyConfiguration: str = ''
    extendedProperties: List[ExtendedPropertyInfo] = ()
    configuredKeyLOV: KeyLOVDefinition2 = None
    shortName: str = ''
    description: str = ''
    metricFormat: FormatPropertiesNX = None
    nonMetricFormat: FormatPropertiesNX = None
    annotation: str = ''
    attributeConfiguration: AttributeConfiguration = None
    arraySize: int = 0
    options: int = 0


@dataclass
class ClassInformation(TcBaseObj):
    """
    Properties of the given class.
    
    :var id: Unique ID of the class.
    :var parent: Class ID of the parent class.
    :var userData2: User data 2 added to this class.
    :var documents: List of attached Icons, Images and NamedRefs to this class.
    :var classAttributesInfo: Class attributes information.
    :var name: Display name of the class.
    :var shortName: Short name
    :var description: Class description
    :var unitSystem: Unit system of the class. See enum ClassUnitSystem for list of valid values.
    :var isAbstract: True value indicates that the class is abstract class, else storage class.
    :var isGroup: True value indicates that the class is a group.
    :var isAssembly: True value indicates that the class is an assembly.
    :var userData1: User data 2 added to this class.
    """
    id: str = ''
    parent: str = ''
    userData2: str = ''
    documents: List[TypedDocument] = ()
    classAttributesInfo: List[ClassAttributeInfo] = ()
    name: str = ''
    shortName: str = ''
    description: str = ''
    unitSystem: ClassUnitSystem = None
    isAbstract: bool = False
    isGroup: bool = False
    isAssembly: bool = False
    userData1: str = ''


@dataclass
class ClassificationObjectInformation(TcBaseObj):
    """
    Structure representing Classification Object details.
    
    :var clsObject: Reference of Classification object.
    :var clsObjId: Unique ID of the Classification object.
    :var classId: Unique ID of the Classification class where this object was created.
    :var unit: Unit system of measure in which the Classification object is stored in.
    :var properties: Array of Classification attributes references that store the properties of this Classification
    object.
    :var relations: References to list of relations.
    """
    clsObject: BusinessObject = None
    clsObjId: str = ''
    classId: str = ''
    unit: UnitSystemType = None
    properties: List[ClassificationProperty] = ()
    relations: List[BusinessObject] = ()


class ClassUnitSystem(Enum):
    """
    &bull;    METRIC_SYSTEM - Metric unit system of measure.
    &bull;    NON_METRIC_SYSTEM  - Non-metric unit system of measure. 
    &bull;    BOTH &ndash; Class supports both unit system.
    """
    METRIC_SYSTEM = 'METRIC_SYSTEM'
    NON_METRIC_SYSTEM = 'NON_METRIC_SYSTEM'
    BOTH = 'BOTH'


class UnitSystemType(Enum):
    """
    UNSPECIFIED_TYPE: No unit system of measure.
    METRIC_TYPE: Metric unit system of measure.
    NON_METRIC_TYPE: Non-metric unit system of measure.
    """
    UNSPECIFIED_TYPE = 'UNSPECIFIED_TYPE'
    METRIC_TYPE = 'METRIC_TYPE'
    NON_METRIC_TYPE = 'NON_METRIC_TYPE'


"""
The list of KeyLOV id and corresponding KeyLOV definition.
"""
KeylovInfoMap = Dict[str, KeyLOVDefinition2]


"""
The map of Classification Class or View ID and corresponding definition.
"""
ClassInfoMap = Dict[str, ClassInformation]


"""
Map of Workspace Object to the details of Classification Object.

&bull;Key - Workspace Object
&bull;Value - ClassificationObject structure representing Classification Object details.
"""
ClassificationObjectsInfoMap = Dict[BusinessObject, List[ClassificationObjectInformation]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from tcsoa.gen.Internal.Core._2010_04.DataManagement import GetSubscribableTypesAndSubtypesResponse
from typing import List
from tcsoa.gen.Internal.Core._2010_04.ProjectLevelSecurity import ProjectSmartFolderHierarchyOutputResponse2
from tcsoa.base import TcService
from tcsoa.gen.Internal.Core._2010_04.StructureManagement import ValidateInStructureAssociationsResponse


class ProjectLevelSecurityService(TcService):

    @classmethod
    def getProjectsSmartFolderHierarchy2(cls, projectIDs: List[str]) -> ProjectSmartFolderHierarchyOutputResponse2:
        """
        This operation returns smart folder hierarchy as configured by the administrator for the given TC_Project
        objects. This operation returns both internal names and corresponding client locale specific display names for
        each project smart folder. If no project with any of the given project IDs exists in the system error code
        101007: the project ID is invalid; will be returned in a partial error.in the SeviceData of the output
        structure.
        """
        return cls.execute_soa_method(
            method_name='getProjectsSmartFolderHierarchy2',
            library='Internal-Core',
            service_date='2010_04',
            service_name='ProjectLevelSecurity',
            params={'projectIDs': projectIDs},
            response_cls=ProjectSmartFolderHierarchyOutputResponse2,
        )


class DataManagementService(TcService):

    @classmethod
    def getSubscribableTypesAndSubTypes(cls, childTypeOption: str) -> GetSubscribableTypesAndSubtypesResponse:
        """
        This operation retrieves  types and subtypes of business objects for which notification subscriptions can be
        created. It returns types names, subtype names and associated display names of those subscribable types based
        on the 'childTypeOption'.
        
        Use cases:
        - Search for subscribable subtypes:
        
        
        It will search for subscribable subtypes when 'childTypeOption' is specified as 'subtype'.
        """
        return cls.execute_soa_method(
            method_name='getSubscribableTypesAndSubTypes',
            library='Internal-Core',
            service_date='2010_04',
            service_name='DataManagement',
            params={'childTypeOption': childTypeOption},
            response_cls=GetSubscribableTypesAndSubtypesResponse,
        )


class StructureManagementService(TcService):

    @classmethod
    def validateInStructureAssociations(cls, associationType: str, bomlines: List[BOMLine], numLevels: int) -> ValidateInStructureAssociationsResponse:
        """
        Given an array of BOMLine business objects and association_type, this function validates the instructure
        association for all Mechatronics core and ESM relations. The associations like ConnectTo, RealizedBy,
        ImplementedBy, Embeds, Dependenton, GatewayOf, or the Signal relations for the given BOMLine business objects
        and its children as per the number of levels are validated. If the relation type is all, validation is done for
        all types of relations. It validates whether any of the existing relation of any of the passed BOMLine business
        objects are violating the business rule. It also validates whether the original context of the relation is
        changed. If so, it adds such relation to the invalid association list and returns the list. If the number of
        levels is zero value, either entire subtree of the selected lines or immediate children of the selected lines
        are processed. Based on relation value, either validation process is done for specific relation or all
        relations.
        
        Use cases:
        When an assembly or product structure is restructured then user might want to call this operation to validate
        the associations. The user can perform restructure operations like insert level, remove level, MoveTo, split
        occurrences, etc. on the structure. Later the user can use this operation to validate for Mechatronics
        relations after performing these operations.
        """
        return cls.execute_soa_method(
            method_name='validateInStructureAssociations',
            library='Internal-Core',
            service_date='2010_04',
            service_name='StructureManagement',
            params={'associationType': associationType, 'bomlines': bomlines, 'numLevels': numLevels},
            response_cls=ValidateInStructureAssociationsResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetDiagnosticInfoForAcctTablesResp(TcBaseObj):
    """
    The return data for getDiagnosticInfoForAcctTables operation.
    
    :var tableInfos: The list of table diagnostic information structures.
    :var serviceData: The service data that includes any partial errors.
    """
    tableInfos: List[TableDiagnosticInfo] = ()
    serviceData: ServiceData = None


@dataclass
class NameValuesInfo(TcBaseObj):
    """
    The information type name and information values structure.
    
    :var infoType: The information type name.
    :var infoValues: The list of information values.
    """
    infoType: str = ''
    infoValues: List[str] = ()


@dataclass
class TableDiagnosticInfo(TcBaseObj):
    """
    The table name and table information structure.
    
    :var tableName: The table name.
    :var tableInfos: The list of table information type name and values.
    """
    tableName: str = ''
    tableInfos: List[NameValuesInfo] = ()
